(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@bravobit/bb-foundation')) :
    typeof define === 'function' && define.amd ? define('@bravobit/bb-foundation/storage', ['exports', '@angular/core', '@bravobit/bb-foundation'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory((global.bravobit = global.bravobit || {}, global.bravobit['bb-foundation'] = global.bravobit['bb-foundation'] || {}, global.bravobit['bb-foundation'].storage = {}), global.ng.core, global.bravobit['bb-foundation']));
}(this, (function (exports, i0, i1) { 'use strict';

    var PolyfillStorage = /** @class */ (function () {
        function PolyfillStorage(_storage) {
            var _this = this;
            this._storage = _storage;
            // State.
            this._isSupported = false;
            this.encode = function (value, attributes) {
                // Make sure the expires attribute is a date and not a number.
                var data = { value: value };
                if (typeof attributes.expires === 'number') {
                    attributes.expires = new Date(attributes.expires);
                }
                if (attributes.expires) {
                    data['expires'] = attributes.expires;
                }
                return JSON.stringify(data);
            };
            this.testIfSupported = function () {
                if (typeof _this._storage === 'undefined') {
                    return false;
                }
                try {
                    var x = '__storage_test__';
                    _this._storage.setItem(x, x);
                    _this._storage.removeItem(x);
                    return true;
                }
                catch (error) {
                    return error instanceof DOMException
                        && (error.code === 22 ||
                            error.code === 1014 ||
                            error.name === 'QuotaExceededError' ||
                            error.name === 'NS_ERROR_DOM_QUOTA_REACHED')
                        && (_this._storage && _this._storage.length !== 0);
                }
            };
            // Check if the storage is supported.
            this._isSupported = this.testIfSupported();
        }
        PolyfillStorage.prototype.get = function (token) {
            // Validate that the storage is supported.
            if (!this._isSupported) {
                return undefined;
            }
            return this.decode(token);
        };
        PolyfillStorage.prototype.set = function (token, value, attributes) {
            // Validate that the storage is supported.
            if (!this._isSupported) {
                return;
            }
            // Stringify the data.
            var rawData = this.encode(value, Object.assign({}, attributes));
            // Set the raw data in the storage.
            this._storage.setItem(token, rawData);
        };
        PolyfillStorage.prototype.remove = function (token) {
            // Validate that the storage is supported.
            if (!this._isSupported) {
                return;
            }
            // Remove the item in the storage.
            this._storage.removeItem(token);
        };
        PolyfillStorage.prototype.clear = function () {
            // Validate that the storage is supported.
            if (!this._isSupported) {
                return;
            }
            // Clear the storage.
            this._storage.clear();
        };
        PolyfillStorage.prototype.isSupported = function () {
            return this._isSupported;
        };
        PolyfillStorage.prototype.decode = function (token) {
            // Retrieve the raw data from the storage.
            var rawData = this._storage.getItem(token);
            // Try to parse the data.
            var parsedData = undefined;
            // Only parse the data when it is not null.
            if (rawData !== null) {
                try {
                    var _b = JSON.parse(rawData), value = _b.value, expires = _b.expires;
                    if (expires !== undefined) {
                        var now = new Date().getTime();
                        var expireDate = new Date(expires).getTime();
                        if (expireDate <= now) {
                            this.remove(token);
                            return undefined;
                        }
                    }
                    parsedData = value;
                }
                catch (_a) {
                    // We dont do anything with failed parsing.
                }
            }
            // Return the parsed data.
            return parsedData;
        };
        return PolyfillStorage;
    }());

    var CookieStorage = /** @class */ (function () {
        function CookieStorage(_isBrowser, _cookieString) {
            var _this = this;
            this._isBrowser = _isBrowser;
            this._cookieString = _cookieString;
            // State.
            this._isSupported = false;
            this.decode = function (cookieString) {
                var result = {};
                var cookies = cookieString ? cookieString.split('; ') : [];
                var rdecode = /(%[0-9A-Z]{2})+/g;
                for (var index = 0; index < cookies.length; index++) {
                    var parts = cookies[index].split('=');
                    var cookie = parts.slice(1).join('=');
                    if (cookie.charAt(0) === '"') {
                        cookie = cookie.slice(1, -1);
                    }
                    try {
                        var name = parts[0].replace(rdecode, decodeURIComponent);
                        var value = cookie.replace(rdecode, decodeURIComponent);
                        result[name] = _this.jsonParse(value);
                    }
                    catch (_a) {
                        // Ignore cookies with an invalid name/value encoding.
                    }
                }
                return result;
            };
            this.jsonParse = function (rawValue) {
                // Try to parse the data.
                var parsedValue = undefined;
                // Only parse the value when it is not null.
                if (rawValue !== undefined) {
                    try {
                        parsedValue = JSON.parse(rawValue);
                    }
                    catch (_a) {
                        // The string could not be parsed to JSON.
                    }
                }
                // Return the parsed value.
                return parsedValue;
            };
            this.stringifyAttribute = function (token, value) {
                if (!value) {
                    return '';
                }
                var stringified = "; " + token;
                // Boolean attributes should not have a value.
                if (value === true) {
                    return stringified;
                }
                return stringified + ("= " + value);
            };
            this.testIfSupported = function () {
                var _a;
                return ((typeof window !== 'undefined') && ((_a = window === null || window === void 0 ? void 0 : window.navigator) === null || _a === void 0 ? void 0 : _a.cookieEnabled)) || false;
            };
            // Check if the storage is supported.
            this._isSupported = this.testIfSupported();
        }
        CookieStorage.prototype.get = function (token) {
            // Get all values and pick one out.
            var values = this.all();
            return values === null || values === void 0 ? void 0 : values[token];
        };
        CookieStorage.prototype.set = function (token, value, attributes) {
            // Validate that the storage is supported.
            if (!this._isSupported) {
                return;
            }
            // Write a new cookie.
            document.cookie = this.encode(token, value, Object.assign({ path: '/', sameSite: 'Lax' }, attributes));
        };
        CookieStorage.prototype.remove = function (token, attributes) {
            // Validate that the storage is supported.
            if (!this._isSupported) {
                return;
            }
            this.set(token, '', Object.assign(Object.assign({}, attributes), { expires: -1 }));
        };
        CookieStorage.prototype.clear = function (attributes) {
            // Validate that the storage is supported.
            if (!this._isSupported) {
                return;
            }
            var items = this.all();
            for (var token in items) {
                this.remove(token, attributes);
            }
        };
        CookieStorage.prototype.isSupported = function () {
            return this._isSupported;
        };
        CookieStorage.prototype.all = function () {
            var cookieString = this._isBrowser
                ? document === null || document === void 0 ? void 0 : document.cookie : this._cookieString;
            return this.decode(cookieString);
        };
        CookieStorage.prototype.encode = function (token, value, attributes) {
            // Encode the token.
            var encodedToken = encodeURIComponent(token)
                .replace(/%(23|24|26|2B|5E|60|7C)/g, decodeURIComponent)
                .replace(/\(/g, '%28').replace(/\)/g, '%29');
            var rawValue = JSON.stringify(value);
            // Encode the value.
            var encodedValue = encodeURIComponent(rawValue)
                .replace(/%(23|24|26|2B|3A|3C|3E|3D|2F|3F|40|5B|5D|5E|60|7B|7D|7C)/g, decodeURIComponent);
            // Return the completed encoded string.
            return [encodedToken, '=', encodedValue, this.stringifyCookie(attributes)].join('');
        };
        CookieStorage.prototype.stringifyCookie = function (attributes) {
            var _a;
            // Make sure the expires attribute is a date and not a number.
            if (typeof attributes.expires === 'number') {
                var expires = new Date();
                expires.setMilliseconds(expires.getMilliseconds() + attributes.expires * 864e+5);
                attributes.expires = expires;
            }
            // Force secure when sameSite=None.
            if (attributes.sameSite === 'None') {
                attributes.secure = true;
                (_a = console === null || console === void 0 ? void 0 : console.warn) === null || _a === void 0 ? void 0 : _a.call(console, 'Cooke was forced with secure flag because sameSite=None');
            }
            // Stringify all attributes.
            return this.stringifyAttribute('Expires', attributes.expires ? attributes.expires.toUTCString() : '')
                + this.stringifyAttribute('Domain', attributes.domain)
                + this.stringifyAttribute('Path', attributes.path)
                + this.stringifyAttribute('Secure', attributes.secure)
                + this.stringifyAttribute('SameSite', attributes.sameSite);
        };
        return CookieStorage;
    }());

    var MemoryStorage = /** @class */ (function () {
        function MemoryStorage() {
            // State.
            this._storage = {};
            this.encode = function (value, attributes) {
                var data = { value: value };
                // Make sure the expires is a date object.
                if (typeof attributes.expires === 'number') {
                    attributes.expires = new Date(attributes.expires);
                }
                // Add the expire date if it was set.
                if (attributes.expires) {
                    data['expires'] = attributes.expires;
                }
                // Return the encoded data.
                return data;
            };
        }
        MemoryStorage.prototype.get = function (token) {
            // Grab the data from the memory storage.
            var result = this._storage[token];
            // Validate the data exists.
            if (result === undefined) {
                return undefined;
            }
            // Extract the value and expire date.
            var value = result.value, expires = result.expires;
            // Validate the value is still valid.
            if (expires !== undefined) {
                var now = new Date().getTime();
                var expireDate = new Date(expires).getTime();
                // Remove the value if it is expired.
                if (expireDate <= now) {
                    this.remove(token);
                    return undefined;
                }
            }
            // Return the value.
            return value;
        };
        MemoryStorage.prototype.set = function (token, value, attributes) {
            // Save the encoded value with the attributes.
            this._storage[token] = this.encode(value, Object.assign({}, attributes));
        };
        MemoryStorage.prototype.remove = function (token) {
            delete this._storage[token];
        };
        MemoryStorage.prototype.clear = function () {
            this._storage = {};
        };
        MemoryStorage.prototype.isSupported = function () {
            // The memory storage is always supported.
            return true;
        };
        return MemoryStorage;
    }());

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation.

    Permission to use, copy, modify, and/or distribute this software for any
    purpose with or without fee is hereby granted.

    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
    PERFORMANCE OF THIS SOFTWARE.
    ***************************************************************************** */
    /* global Reflect, Promise */
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b)
                if (Object.prototype.hasOwnProperty.call(b, p))
                    d[p] = b[p]; };
        return extendStatics(d, b);
    };
    function __extends(d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    }
    var __assign = function () {
        __assign = Object.assign || function __assign(t) {
            for (var s, i = 1, n = arguments.length; i < n; i++) {
                s = arguments[i];
                for (var p in s)
                    if (Object.prototype.hasOwnProperty.call(s, p))
                        t[p] = s[p];
            }
            return t;
        };
        return __assign.apply(this, arguments);
    };
    function __rest(s, e) {
        var t = {};
        for (var p in s)
            if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
                t[p] = s[p];
        if (s != null && typeof Object.getOwnPropertySymbols === "function")
            for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
                if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                    t[p[i]] = s[p[i]];
            }
        return t;
    }
    function __decorate(decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function")
            r = Reflect.decorate(decorators, target, key, desc);
        else
            for (var i = decorators.length - 1; i >= 0; i--)
                if (d = decorators[i])
                    r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    }
    function __param(paramIndex, decorator) {
        return function (target, key) { decorator(target, key, paramIndex); };
    }
    function __metadata(metadataKey, metadataValue) {
        if (typeof Reflect === "object" && typeof Reflect.metadata === "function")
            return Reflect.metadata(metadataKey, metadataValue);
    }
    function __awaiter(thisArg, _arguments, P, generator) {
        function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
        return new (P || (P = Promise))(function (resolve, reject) {
            function fulfilled(value) { try {
                step(generator.next(value));
            }
            catch (e) {
                reject(e);
            } }
            function rejected(value) { try {
                step(generator["throw"](value));
            }
            catch (e) {
                reject(e);
            } }
            function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
            step((generator = generator.apply(thisArg, _arguments || [])).next());
        });
    }
    function __generator(thisArg, body) {
        var _ = { label: 0, sent: function () { if (t[0] & 1)
                throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
        return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function () { return this; }), g;
        function verb(n) { return function (v) { return step([n, v]); }; }
        function step(op) {
            if (f)
                throw new TypeError("Generator is already executing.");
            while (_)
                try {
                    if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done)
                        return t;
                    if (y = 0, t)
                        op = [op[0] & 2, t.value];
                    switch (op[0]) {
                        case 0:
                        case 1:
                            t = op;
                            break;
                        case 4:
                            _.label++;
                            return { value: op[1], done: false };
                        case 5:
                            _.label++;
                            y = op[1];
                            op = [0];
                            continue;
                        case 7:
                            op = _.ops.pop();
                            _.trys.pop();
                            continue;
                        default:
                            if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) {
                                _ = 0;
                                continue;
                            }
                            if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) {
                                _.label = op[1];
                                break;
                            }
                            if (op[0] === 6 && _.label < t[1]) {
                                _.label = t[1];
                                t = op;
                                break;
                            }
                            if (t && _.label < t[2]) {
                                _.label = t[2];
                                _.ops.push(op);
                                break;
                            }
                            if (t[2])
                                _.ops.pop();
                            _.trys.pop();
                            continue;
                    }
                    op = body.call(thisArg, _);
                }
                catch (e) {
                    op = [6, e];
                    y = 0;
                }
                finally {
                    f = t = 0;
                }
            if (op[0] & 5)
                throw op[1];
            return { value: op[0] ? op[1] : void 0, done: true };
        }
    }
    var __createBinding = Object.create ? (function (o, m, k, k2) {
        if (k2 === undefined)
            k2 = k;
        Object.defineProperty(o, k2, { enumerable: true, get: function () { return m[k]; } });
    }) : (function (o, m, k, k2) {
        if (k2 === undefined)
            k2 = k;
        o[k2] = m[k];
    });
    function __exportStar(m, o) {
        for (var p in m)
            if (p !== "default" && !Object.prototype.hasOwnProperty.call(o, p))
                __createBinding(o, m, p);
    }
    function __values(o) {
        var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
        if (m)
            return m.call(o);
        if (o && typeof o.length === "number")
            return {
                next: function () {
                    if (o && i >= o.length)
                        o = void 0;
                    return { value: o && o[i++], done: !o };
                }
            };
        throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }
    function __read(o, n) {
        var m = typeof Symbol === "function" && o[Symbol.iterator];
        if (!m)
            return o;
        var i = m.call(o), r, ar = [], e;
        try {
            while ((n === void 0 || n-- > 0) && !(r = i.next()).done)
                ar.push(r.value);
        }
        catch (error) {
            e = { error: error };
        }
        finally {
            try {
                if (r && !r.done && (m = i["return"]))
                    m.call(i);
            }
            finally {
                if (e)
                    throw e.error;
            }
        }
        return ar;
    }
    /** @deprecated */
    function __spread() {
        for (var ar = [], i = 0; i < arguments.length; i++)
            ar = ar.concat(__read(arguments[i]));
        return ar;
    }
    /** @deprecated */
    function __spreadArrays() {
        for (var s = 0, i = 0, il = arguments.length; i < il; i++)
            s += arguments[i].length;
        for (var r = Array(s), k = 0, i = 0; i < il; i++)
            for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
                r[k] = a[j];
        return r;
    }
    function __spreadArray(to, from) {
        for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
            to[j] = from[i];
        return to;
    }
    function __await(v) {
        return this instanceof __await ? (this.v = v, this) : new __await(v);
    }
    function __asyncGenerator(thisArg, _arguments, generator) {
        if (!Symbol.asyncIterator)
            throw new TypeError("Symbol.asyncIterator is not defined.");
        var g = generator.apply(thisArg, _arguments || []), i, q = [];
        return i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i;
        function verb(n) { if (g[n])
            i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; }
        function resume(n, v) { try {
            step(g[n](v));
        }
        catch (e) {
            settle(q[0][3], e);
        } }
        function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }
        function fulfill(value) { resume("next", value); }
        function reject(value) { resume("throw", value); }
        function settle(f, v) { if (f(v), q.shift(), q.length)
            resume(q[0][0], q[0][1]); }
    }
    function __asyncDelegator(o) {
        var i, p;
        return i = {}, verb("next"), verb("throw", function (e) { throw e; }), verb("return"), i[Symbol.iterator] = function () { return this; }, i;
        function verb(n, f) { i[n] = o[n] ? function (v) { return (p = !p) ? { value: __await(o[n](v)), done: n === "return" } : f ? f(v) : v; } : f; }
    }
    function __asyncValues(o) {
        if (!Symbol.asyncIterator)
            throw new TypeError("Symbol.asyncIterator is not defined.");
        var m = o[Symbol.asyncIterator], i;
        return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
        function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
        function settle(resolve, reject, d, v) { Promise.resolve(v).then(function (v) { resolve({ value: v, done: d }); }, reject); }
    }
    function __makeTemplateObject(cooked, raw) {
        if (Object.defineProperty) {
            Object.defineProperty(cooked, "raw", { value: raw });
        }
        else {
            cooked.raw = raw;
        }
        return cooked;
    }
    ;
    var __setModuleDefault = Object.create ? (function (o, v) {
        Object.defineProperty(o, "default", { enumerable: true, value: v });
    }) : function (o, v) {
        o["default"] = v;
    };
    function __importStar(mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k))
                    __createBinding(result, mod, k);
        __setModuleDefault(result, mod);
        return result;
    }
    function __importDefault(mod) {
        return (mod && mod.__esModule) ? mod : { default: mod };
    }
    function __classPrivateFieldGet(receiver, privateMap) {
        if (!privateMap.has(receiver)) {
            throw new TypeError("attempted to get private field on non-instance");
        }
        return privateMap.get(receiver);
    }
    function __classPrivateFieldSet(receiver, privateMap, value) {
        if (!privateMap.has(receiver)) {
            throw new TypeError("attempted to set private field on non-instance");
        }
        privateMap.set(receiver, value);
        return value;
    }

    var Storage = /** @class */ (function () {
        function Storage(_platform, _cookieString) {
            this._platform = _platform;
            this._cookieString = _cookieString;
            // Current selected storage strategy.
            this._current = null;
            this.getLocalStorage = function () {
                return (typeof window !== 'undefined')
                    ? window === null || window === void 0 ? void 0 : window.localStorage : null;
            };
            this.getSessionStorage = function () {
                return (typeof window !== 'undefined')
                    ? window === null || window === void 0 ? void 0 : window.sessionStorage : null;
            };
            this._strategies = {
                local: new PolyfillStorage(this.getLocalStorage()),
                cookie: new CookieStorage(this._platform.isBrowser, this._cookieString),
                session: new PolyfillStorage(this.getSessionStorage()),
                memory: new MemoryStorage()
            };
            this._current = this.findBestStorageStrategy();
        }
        Object.defineProperty(Storage.prototype, "local", {
            get: function () {
                return this._strategies.local;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(Storage.prototype, "cookie", {
            get: function () {
                return this._strategies.cookie;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(Storage.prototype, "session", {
            get: function () {
                return this._strategies.session;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(Storage.prototype, "memory", {
            get: function () {
                return this._strategies.memory;
            },
            enumerable: false,
            configurable: true
        });
        Storage.prototype.get = function (token) {
            return this._current.get(token);
        };
        Storage.prototype.set = function (token, value, attributes) {
            this._current.set(token, value, attributes);
        };
        Storage.prototype.remove = function (token, attributes) {
            this._current.remove(token, attributes);
        };
        Storage.prototype.clear = function (attributes) {
            this._current.clear(attributes);
        };
        Storage.prototype.isSupported = function () {
            return !!this._current;
        };
        Storage.prototype.findBestStorageStrategy = function () {
            var e_1, _a;
            var types = Object.keys(this._strategies);
            try {
                // Loop through all available strategies to find the best.
                for (var types_1 = __values(types), types_1_1 = types_1.next(); !types_1_1.done; types_1_1 = types_1.next()) {
                    var type = types_1_1.value;
                    var strategy = this._strategies[type];
                    // The first supported strategy is the best.
                    if (strategy.isSupported()) {
                        return strategy;
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (types_1_1 && !types_1_1.done && (_a = types_1.return)) _a.call(types_1);
                }
                finally { if (e_1) throw e_1.error; }
            }
            // If no strategy was returned we emit an error.
            throw new Error('No storage strategy was available.');
        };
        return Storage;
    }());
    Storage.ɵprov = i0.ɵɵdefineInjectable({ factory: function Storage_Factory() { return new Storage(i0.ɵɵinject(i1.Platform), i0.ɵɵinject(i1.COOKIE, 8)); }, token: Storage, providedIn: "root" });
    Storage.decorators = [
        { type: i0.Injectable, args: [{
                    providedIn: 'root'
                },] }
    ];
    Storage.ctorParameters = function () { return [
        { type: i1.Platform },
        { type: String, decorators: [{ type: i0.Optional }, { type: i0.Inject, args: [i1.COOKIE,] }] }
    ]; };

    /**
     * Generated bundle index. Do not edit.
     */

    exports.CookieStorage = CookieStorage;
    exports.MemoryStorage = MemoryStorage;
    exports.PolyfillStorage = PolyfillStorage;
    exports.Storage = Storage;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=bravobit-bb-foundation-storage.umd.js.map
