import { Component, ChangeDetectionStrategy, ViewEncapsulation, ElementRef, HostListener, Directive, Optional, Host, Input, InjectionToken, EventEmitter, forwardRef, ViewChild, Output, ContentChild, Inject, LOCALE_ID, Self, HostBinding, TemplateRef, ViewContainerRef, Renderer2, Pipe, ChangeDetectorRef, NgZone, NgModule } from '@angular/core';
import { trigger, transition, style, animate } from '@angular/animations';
import { BehaviorSubject, Subscription, combineLatest, of, fromEvent, merge, EMPTY } from 'rxjs';
import { __awaiter } from 'tslib';
import { mixinDisabled, mixinGrouped, ImageConverter, mixinFocused, mixinReadonly, mixinRequired, mixinError, mixinHideErrors, coerceBooleanProperty, Platform, mixinLoad } from '@bravobit/bb-foundation';
import { NG_VALUE_ACCESSOR, NgControl, NG_VALIDATORS, Validators, FormGroup, FormControl, FormGroupDirective, ReactiveFormsModule } from '@angular/forms';
import { map, distinctUntilChanged, tap, startWith, mapTo, shareReplay, delay } from 'rxjs/operators';
import { formatDate, DatePipe, CommonModule } from '@angular/common';
import { Localize, LocalizeModule } from '@bravobit/bb-foundation/localize';
import { DomSanitizer } from '@angular/platform-browser';

class BbDropdown {
    constructor(_elementRef) {
        this._elementRef = _elementRef;
        // State.
        this.visible$ = new BehaviorSubject(false);
    }
    toggle() {
        return this.visible$.next(!this.visible$.getValue());
    }
    close() {
        return this.visible$.next(false);
    }
    onDocumentClick(target) {
        var _a, _b;
        if (!target) {
            return;
        }
        if ((_b = (_a = this._elementRef) === null || _a === void 0 ? void 0 : _a.nativeElement) === null || _b === void 0 ? void 0 : _b.contains(target)) {
            return;
        }
        return this.close();
    }
}
BbDropdown.decorators = [
    { type: Component, args: [{
                selector: 'bb-dropdown',
                template: "<button (click)=\"toggle()\"\n        [class.opened]=\"visible$ | async\"\n        type=\"button\">\n    <ng-content></ng-content>\n</button>\n\n<div *ngIf=\"visible$ | async\"\n     [@dropdownMenuAnimation]=\"true\"\n     class=\"bb-dropdown-menu\"\n     role=\"list\">\n    <ng-content select=\"[bbDropdownItem], hr\"></ng-content>\n</div>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                host: { 'class': 'bb-dropdown' },
                preserveWhitespaces: false,
                animations: [
                    trigger('dropdownMenuAnimation', [
                        transition(':enter', [
                            // 1. Set the initial state.
                            style({ opacity: 0, transform: 'scale(0.9)' }),
                            // 2. Start the animation to show the item.
                            animate('80ms cubic-bezier(0, 0, .2, 1)', style({ opacity: 1, transform: 'scale(1)' }))
                        ]),
                        transition(':leave', [
                            // 1. Set the initial state.
                            style({ opacity: 1, transform: 'scale(1)' }),
                            // 2. Start the animation to hide the item.
                            animate('80ms cubic-bezier(0, 0, .2, 1)', style({ opacity: 0, transform: 'scale(0.9)' }))
                        ])
                    ])
                ],
                styles: [".bb-dropdown{display:inline-block;position:relative}.bb-dropdown.right>.bb-dropdown-menu{left:auto;right:0;transform-origin:top right}.bb-dropdown-menu{background-color:#fff;border:1px solid #ccc;border-radius:4px;box-shadow:0 0 6px 0 rgba(0,0,0,.1);display:block;left:0;list-style-type:none;margin:10px 0 0;padding:0;position:absolute;top:100%;transform-origin:top left;z-index:10}.bb-dropdown-menu:after{border-color:transparent transparent #d9d9d9;border-style:solid;border-width:0 10px 5px;content:\"\";height:0;position:absolute;right:5px;top:-5px;width:0}.bb-dropdown-menu>hr{background-color:#ccc;border:none;height:1px;margin:0}.bb-dropdown-item{-moz-user-select:none;-ms-user-select:none;-webkit-user-select:none;background-color:transparent;border:none;color:#676767!important;cursor:pointer;display:block;font-weight:500;line-height:1;padding:12px 20px;text-align:left;text-decoration:none;transition:background-color .25s cubic-bezier(0,0,.2,1);user-select:none;white-space:nowrap;width:100%}.bb-dropdown-item:first-child{border-top-left-radius:4px;border-top-right-radius:4px}.bb-dropdown-item:hover{background-color:#f6f6f6}.bb-dropdown-item:focus{box-shadow:0 0 0 3px rgba(220,224,233,.5)}.bb-dropdown-item:active{background-color:#f0f0f0}"]
            },] }
];
BbDropdown.ctorParameters = () => [
    { type: ElementRef }
];
BbDropdown.propDecorators = {
    onDocumentClick: [{ type: HostListener, args: ['document:click', ['$event.target'],] }]
};
class BbDropdownItem {
    constructor(_parent) {
        this._parent = _parent;
        // Inputs.
        this.closeOnClick = true;
    }
    onClick() {
        var _a;
        if (!this.closeOnClick) {
            return;
        }
        (_a = this._parent) === null || _a === void 0 ? void 0 : _a.close();
    }
}
BbDropdownItem.decorators = [
    { type: Directive, args: [{
                selector: '[bbDropdownItem]',
                host: {
                    'class': 'bb-dropdown-item'
                }
            },] }
];
BbDropdownItem.ctorParameters = () => [
    { type: BbDropdown, decorators: [{ type: Optional }, { type: Host }] }
];
BbDropdownItem.propDecorators = {
    closeOnClick: [{ type: Input }],
    onClick: [{ type: HostListener, args: ['click',] }]
};

const ELEMENTS_ICONS = new InjectionToken('elements_icons');
const ELEMENTS_ERRORS = new InjectionToken('elements_errors');

class BbImagePickerBase {
}
const BbImagePickerMixinBase = mixinDisabled(mixinGrouped(BbImagePickerBase));
class BbImagePicker extends BbImagePickerMixinBase {
    constructor(_converter) {
        super();
        this._converter = _converter;
        // Inputs.
        this.label = null;
        this.placeholder = null;
        this.styleDimensions = null;
        this.cropDimensions = null;
        // Form.
        this._value$ = new BehaviorSubject(null);
        this.valueChange = new EventEmitter();
        this.onTouchedCallback = () => {
        };
        this.onChangeCallback = () => {
        };
    }
    get value() {
        return this._value$.getValue();
    }
    set value(value) {
        this._value$.next(value);
        this.valueChange.emit(value);
    }
    get image() {
        return this._value$.pipe(map(value => value || this.placeholder), map(value => value ? `url(${value})` : null));
    }
    onFileChange(event) {
        return __awaiter(this, void 0, void 0, function* () {
            // Grab the file from the event.
            const file = event.target.files[0];
            // Check if the file exists.
            if (!file || this.disabled) {
                return;
            }
            try {
                this.value = yield this._converter.toDataUri(file, this.cropDimensions);
            }
            catch (error) {
                this.value = null;
            }
            this.onChangeCallback(this.value);
            if (this.fileInput && this.fileInput.nativeElement) {
                this.fileInput.nativeElement.value = '';
            }
        });
    }
    openFilePicker() {
        if (this.disabled) {
            return;
        }
        // Execute a click on the file input.
        this.fileInput
            && this.fileInput.nativeElement
            && this.fileInput.nativeElement.click
            && this.fileInput.nativeElement.click();
    }
    writeValue(value) {
        this.value = value;
    }
    registerOnChange(method) {
        this.onChangeCallback = method;
    }
    registerOnTouched(method) {
        this.onTouchedCallback = method;
    }
    setDisabledState(isDisabled) {
        this.disabled = isDisabled;
    }
}
BbImagePicker.decorators = [
    { type: Component, args: [{
                selector: 'bb-image-picker',
                template: "<!-- The input that can open the file picker. -->\n<input #fileInput\n       (change)=\"onFileChange($event)\"\n       class=\"bb-image-picker-input\"\n       type=\"file\"\n       tabindex=\"-1\"\n       accept=\"image/*\">\n\n<span [style.width]=\"styleDimensions?.width || '100%'\"\n      [style.height]=\"styleDimensions?.height || '215px'\"\n      [style.background-image]=\"image | async\"\n      class=\"bb-image-input-image\">\n    </span>\n\n<button *ngIf=\"label as labelText\"\n        (click)=\"openFilePicker()\"\n        bb-button\n        class=\"bb-image-input-button\"\n        type=\"button\">\n    {{ labelText }}\n</button>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                host: {
                    'class': 'bb-image-picker',
                    '[class.disabled]': 'disabled',
                    '[class.grouped]': 'grouped'
                },
                inputs: ['grouped', 'disabled'],
                providers: [{ provide: NG_VALUE_ACCESSOR, useExisting: forwardRef(() => BbImagePicker), multi: true }],
                preserveWhitespaces: false,
                styles: [".bb-image-picker{align-items:center;display:flex;flex-direction:column}.bb-image-picker.circle>.bb-image-input-image{border-radius:50%}.bb-image-picker.grouped{margin-bottom:20px}.bb-image-picker.disabled{-moz-user-select:none;-ms-user-select:none;-webkit-user-select:none;cursor:default;opacity:.5;pointer-events:none;user-select:none}.bb-image-picker-input{height:.1px;opacity:0;overflow:hidden;position:absolute;width:.1px;z-index:-1}.bb-image-input-image{align-items:center;background:rgba(0,0,0,.2) url(data:image/gif;base64,R0lGODlhAQABAIAAAAAAAP///yH5BAEAAAAALAAAAAABAAEAAAIBRAA7) 50%/cover no-repeat;border-radius:4px;cursor:default;display:flex;justify-content:center;position:relative}.bb-image-input-button{margin-top:10px}"]
            },] }
];
BbImagePicker.ctorParameters = () => [
    { type: ImageConverter }
];
BbImagePicker.propDecorators = {
    fileInput: [{ type: ViewChild, args: ['fileInput', { static: true },] }],
    label: [{ type: Input }],
    placeholder: [{ type: Input }],
    styleDimensions: [{ type: Input }],
    cropDimensions: [{ type: Input }],
    value: [{ type: Input }],
    valueChange: [{ type: Output }]
};

let nextUniqueId = 0;
class BbFormControlBase {
}
const BbFormControlMixinBase = mixinDisabled(mixinFocused(mixinReadonly(mixinGrouped(mixinRequired(mixinError(mixinHideErrors(BbFormControlBase)))))));
class BbFormControl extends BbFormControlMixinBase {
    constructor() {
        super(...arguments);
        // Inputs.
        this.label = null;
        this.hint = null;
        // Data.
        this.arrow = false;
        // Data.
        this.labelId = `bb-form-control-${nextUniqueId++}`;
    }
    onErrorChange(error) {
        this.error = !!error;
    }
}
BbFormControl.decorators = [
    { type: Component, args: [{
                selector: 'bb-form-control',
                template: "<!-- Label of the form control. -->\n<label *ngIf=\"label as labelContent\"\n       [attr.for]=\"labelId\"\n       class=\"bb-form-control-label\">\n    <ng-template [bbTemplate]=\"labelContent\">\n        {{ labelContent }}\n    </ng-template>\n</label>\n\n<!-- The input with the addons. -->\n<div class=\"bb-form-control-container\">\n    <ng-content select=\"[bbPrefix]\"></ng-content>\n    <ng-content select=\"[bbInput]\"></ng-content>\n    <ng-content select=\"[bbSuffix]\"></ng-content>\n</div>\n\n<!-- The error component. -->\n<bb-form-error *ngIf=\"!hideErrors\"\n               [control]=\"ngControl\"\n               (errorChange)=\"onErrorChange($event)\">\n</bb-form-error>\n\n<!-- The form control hint. -->\n<p *ngIf=\"hint as hintContent\"\n   class=\"bb-form-control-hint\">\n    <ng-template [bbTemplate]=\"hintContent\">\n        {{ hintContent }}\n    </ng-template>\n</p>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                host: {
                    'class': 'bb-form-control',
                    '[class.readonly]': 'readonly',
                    '[class.required]': 'required',
                    '[class.disabled]': 'disabled',
                    '[class.focused]': 'focused',
                    '[class.grouped]': 'grouped',
                    '[class.arrow]': 'arrow',
                    '[class.error]': 'error'
                },
                inputs: ['grouped', 'hideErrors'],
                preserveWhitespaces: false,
                styles: [".bb-form-control{color:#2d3c4d;display:block}.bb-form-control>.bb-form-control-container>.bb-prefix,.bb-form-control>.bb-form-control-container>.bb-suffix{color:#b4b4b4;margin:0 8px;transition:color .3s cubic-bezier(0,0,.2,1)}.bb-form-control.grouped{margin-bottom:20px}.bb-form-control.small{max-width:240px}.bb-form-control.medium{max-width:480px}.bb-form-control.large{max-width:720px}.bb-form-control.required>.bb-form-control-label:after{color:#c23934;content:\"*\";font-size:12px;line-height:1.5}.bb-form-control.disabled>.bb-form-control-container{background-color:hsla(0,0%,80%,.64);cursor:default;pointer-events:none}.bb-form-control.disabled>.bb-form-control-container>input,.bb-form-control.disabled>.bb-form-control-container>select,.bb-form-control.disabled>.bb-form-control-container>textarea{color:#aaa}.bb-form-control.readonly>.bb-form-control-container{border:1px dotted #bdc4c9;cursor:default}.bb-form-control:not(.disabled):not(.readonly).focused>.bb-form-control-container{background-color:#fff;box-shadow:inset 0 3px 0 0 rgba(183,188,194,.2),0 6px 6px -6px rgba(0,0,0,.1),0 0 0 3px rgba(0,123,255,.2)}.bb-form-control:not(.disabled):not(.readonly).focused>.bb-form-control-container>input::-moz-placeholder,.bb-form-control:not(.disabled):not(.readonly).focused>.bb-form-control-container>textarea::-moz-placeholder{-moz-transition:color .3s cubic-bezier(0,0,.2,1);color:#b4b4b4;transition:color .3s cubic-bezier(0,0,.2,1)}.bb-form-control:not(.disabled):not(.readonly).focused>.bb-form-control-container>input:-ms-input-placeholder,.bb-form-control:not(.disabled):not(.readonly).focused>.bb-form-control-container>textarea:-ms-input-placeholder{-ms-transition:color .3s cubic-bezier(0,0,.2,1);color:#b4b4b4;transition:color .3s cubic-bezier(0,0,.2,1)}.bb-form-control:not(.disabled):not(.readonly).focused>.bb-form-control-container>input::placeholder,.bb-form-control:not(.disabled):not(.readonly).focused>.bb-form-control-container>textarea::placeholder{color:#b4b4b4;transition:color .3s cubic-bezier(0,0,.2,1)}.bb-form-control.error>.bb-form-control-label{color:#c23934}.bb-form-control.error>.bb-form-control-hint{display:none}.bb-form-control.error>.bb-form-control-container{background-color:rgba(194,57,52,.25)!important;border:1px solid #962b26;box-shadow:inset 0 3px 0 0 rgba(183,188,194,.2),0 6px 6px -6px rgba(0,0,0,.1),0 0 0 3px rgba(194,57,52,.4)!important}.bb-form-control.error>.bb-form-control-container>.bb-prefix,.bb-form-control.error>.bb-form-control-container>.bb-suffix{color:#c23934}.bb-form-control.error>.bb-form-control-container>input::-moz-placeholder,.bb-form-control.error>.bb-form-control-container>textarea::-moz-placeholder{color:rgba(194,57,52,.4)!important}.bb-form-control.error>.bb-form-control-container>input:-ms-input-placeholder,.bb-form-control.error>.bb-form-control-container>textarea:-ms-input-placeholder{color:rgba(194,57,52,.4)!important}.bb-form-control.error>.bb-form-control-container>input::placeholder,.bb-form-control.error>.bb-form-control-container>textarea::placeholder{color:rgba(194,57,52,.4)!important}.bb-form-control.arrow>.bb-form-control-container:after{background-image:url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAA0AAAAGCAYAAAAYLBS/AAAACXBIWXMAAAsSAAALEgHS3X78AAAAWUlEQVQYlY3QsQ2DMAAEwMMTsAkrZInvWY3WXiIjJJtkA1OBRCIRf/nSffFTrXXGE4uxbCXJBw+8R0CStcAg3JKsUI7mDzzBBd3AC4Cp9/4z21o7znl9A9gBAf4fpjLTwkUAAAAASUVORK5CYII=);background-position:calc(100% - 8px) 50%;background-repeat:no-repeat;bottom:0;content:\"\";display:flex;height:36px;min-width:20px;pointer-events:none;position:absolute;right:0;top:0;width:20px}.bb-form-control-container{align-items:center;background-color:#fff8ff;border:1px solid #b6bbc1;border-radius:3px;box-shadow:inset 0 3px 0 0 rgba(183,188,194,.2),0 6px 6px -6px rgba(0,0,0,.1);display:flex;overflow:hidden;position:relative;transition-duration:.25s;transition-property:background-color,box-shadow;transition-timing-function:cubic-bezier(0,0,.2,1);width:100%}.bb-form-control-container>input::-moz-placeholder,.bb-form-control-container>textarea::-moz-placeholder{-moz-transition:color .3s cubic-bezier(0,0,.2,1);color:#d2d2d2;transition:color .3s cubic-bezier(0,0,.2,1)}.bb-form-control-container>input:-ms-input-placeholder,.bb-form-control-container>textarea:-ms-input-placeholder{-ms-transition:color .3s cubic-bezier(0,0,.2,1);color:#d2d2d2;transition:color .3s cubic-bezier(0,0,.2,1)}.bb-form-control-container>input::placeholder,.bb-form-control-container>textarea::placeholder{color:#d2d2d2;transition:color .3s cubic-bezier(0,0,.2,1)}.bb-form-control-container>input,.bb-form-control-container>select,.bb-form-control-container>textarea{-moz-appearance:none;-webkit-appearance:none;appearance:none;background-color:transparent;border:none;color:#3d464d;font-size:16px;margin:0;width:100%}.bb-form-control-container>input,.bb-form-control-container>select{height:36px;padding:0 8px}.bb-form-control-container>select{padding-right:28px}.bb-form-control-container>select>::-ms-expand{display:none}.bb-form-control-container>input::-ms-clear,.bb-form-control-container>input::-ms-reveal{display:none;height:0;width:0}.bb-form-control-container>input::-webkit-search-cancel-button,.bb-form-control-container>input::-webkit-search-decoration,.bb-form-control-container>input::-webkit-search-results-button,.bb-form-control-container>input::-webkit-search-results-decoration{display:none}.bb-form-control-container>textarea{padding:8px;resize:none}.bb-form-control-label{color:#000;display:block;margin-bottom:4px;position:relative;transition:color 30ms cubic-bezier(0,0,.2,1)}.bb-form-control-error,.bb-form-control-hint{display:block;font-size:13px;margin-top:4px}.bb-form-control-error{color:#c23934}.bb-form-control-hint{color:#738694}"]
            },] }
];
BbFormControl.propDecorators = {
    ngControl: [{ type: ContentChild, args: [NgControl,] }],
    label: [{ type: Input }],
    hint: [{ type: Input }]
};

let nextUniqueId$1 = 0;
class BbFilePickerBase {
}
const BbFilePickerMixinBase = mixinRequired(mixinDisabled(mixinGrouped(mixinError(mixinHideErrors(BbFilePickerBase)))));
class BbFilePicker extends BbFilePickerMixinBase {
    constructor() {
        super(...arguments);
        // Inputs.
        this.label = null;
        this.hint = null;
        this.accept = null;
        this.showImages = true;
        this.allowDragging = true;
        // State.
        this.labelId = `bb-file-picker-${nextUniqueId$1++}`;
        this.hovered = false;
        this.value$ = new BehaviorSubject({});
        this.valueChange = new EventEmitter();
        this.onTouchedCallback = () => {
        };
        this.onChangeCallback = () => {
        };
        this.saveFile = (files) => {
            // Validate the files exist.
            if (!files) {
                return;
            }
            // Retrieve the first file.
            const file = files[0];
            // Validate the first file exists.
            if (!file) {
                return;
            }
            // Save the file.
            this.value = file;
        };
        this.getFileListFromEvent = (event) => {
            const element = event.target;
            if (!element) {
                return null;
            }
            return element.files;
        };
        this.preventDefault = (event) => {
            event.preventDefault();
            event.stopPropagation();
        };
    }
    get value() {
        const { file } = this.value$.getValue();
        return file;
    }
    set value(value) {
        this.value$.next({ file: value });
        this.valueChange.emit(value);
    }
    onDrag(event, isHovered) {
        // Validate the input can support drag.
        if (!this.allowDragging || this.disabled) {
            return;
        }
        // Prevent the default event action
        // and start the hovering.
        this.preventDefault(event);
        this.hovered = isHovered;
    }
    onDrop(event) {
        // Validate the input can support drag.
        if (!this.allowDragging || this.disabled) {
            return;
        }
        // Prevent the default event action
        // and stop the hovering.
        this.preventDefault(event);
        this.hovered = false;
        // Save the file.
        this.saveFile(event['dataTransfer'].files);
        // Run the on change callback.
        this.onChangeCallback(this.value);
    }
    onButtonPressed(file) {
        if (file) {
            this.value = null;
            this.onChangeCallback(this.value);
            return;
        }
        return this.openFilePicker();
    }
    openFilePicker() {
        return this.fileInput
            && this.fileInput.nativeElement
            && this.fileInput.nativeElement.click
            && this.fileInput.nativeElement.click();
    }
    onFileChange(event) {
        // Validate the input is not disabled.
        if (this.disabled) {
            return;
        }
        // Extract the file from the event.
        const fileList = this.getFileListFromEvent(event);
        // Save the file.
        this.saveFile(fileList);
        // Run the on change callback.
        this.onChangeCallback(this.value);
        // Clear the file input.
        if (this.fileInput && this.fileInput.nativeElement) {
            this.fileInput.nativeElement.value = '';
        }
    }
    writeValue(value) {
        this.value = value;
    }
    registerOnChange(method) {
        this.onChangeCallback = method;
    }
    registerOnTouched(method) {
        this.onTouchedCallback = method;
    }
    setDisabledState(isDisabled) {
        this.disabled = isDisabled;
    }
    onErrorChange(error) {
        this.error = !!error;
    }
    validate({ value }) {
        if (value === null || value === undefined) {
            return null;
        }
        const regexString = (this.accept || '*')
            .replace(/\*/g, '.\*')
            .replace(/,/g, '|');
        const mimeTypeRegex = new RegExp(regexString);
        const isNotValid = (typeof File === 'undefined') || !(value instanceof File) || !mimeTypeRegex.test(value === null || value === void 0 ? void 0 : value.type);
        return isNotValid && {
            invalidFileType: true
        };
    }
}
BbFilePicker.decorators = [
    { type: Component, args: [{
                selector: 'bb-file-picker',
                template: "<!-- The label of the input. -->\n<label *ngIf=\"label as labelContent\"\n       [for]=\"labelId\"\n       class=\"bb-file-picker-label\">\n    <ng-template [bbTemplate]=\"labelContent\">\n        {{ labelContent }}\n    </ng-template>\n</label>\n\n<!-- The input that can open the file picker. -->\n<input #fileInput\n       [accept]=\"accept\"\n       [disabled]=\"disabled\"\n       (change)=\"onFileChange($event)\"\n       class=\"bb-file-picker-input\"\n       type=\"file\"\n       tabindex=\"-1\">\n\n<div *ngIf=\"value$ | async as data\"\n     class=\"bb-file-picker-wrapper\">\n    <!-- The button that accepts files. -->\n    <div class=\"bb-file-picker-container\">\n        <button [class.destructive]=\"!!data?.file\"\n                [class.standalone]=\"!allowDragging\"\n                [disabled]=\"disabled\"\n                [id]=\"labelId\"\n                (click)=\"onButtonPressed(data?.file)\"\n                type=\"button\"\n                class=\"bb-file-picker-button\">\n            <ng-container *ngTemplateOutlet=\"data?.file ? closeIcon : uploadIcon\"></ng-container>\n            {{ (data?.file ? 'file-picker.remove' : 'file-picker.choose') | bbLocalize }}\n        </button>\n        <div *ngIf=\"allowDragging\"\n             class=\"bb-file-picker-zone\">\n            {{ 'file-picker.drop' | bbLocalize }}\n        </div>\n    </div>\n\n    <ng-container *ngIf=\"!!extraTemplate\">\n        <ng-container *ngTemplateOutlet=\"extraTemplate; context: {$implicit: data?.file}\"></ng-container>\n    </ng-container>\n\n    <ng-container *ngIf=\"showImages\">\n        <div *ngIf=\"data?.file | bbFileImage | async as image\"\n             class=\"bb-file-picker-image-container\">\n            <div class=\"bb-file-picker-image-wrapper\">\n                <div [@bbFilePickerImageAnimation]=\"true\"\n                     [style.padding-top.%]=\"image?.aspectRatio * 100\"\n                     [style.background-image]=\"image?.background\"\n                     class=\"bb-file-picker-image\">\n                </div>\n            </div>\n        </div>\n    </ng-container>\n\n    <!-- Extra information of the file. -->\n    <div [class.visible]=\"!!data?.file\"\n         class=\"bb-file-picker-info\">\n        <span *ngIf=\"data?.file?.name as name\">{{ name }}</span>\n        <span *ngIf=\"data?.file?.size as size\">{{ size | bbFileSize }}</span>\n    </div>\n</div>\n\n<bb-form-error *ngIf=\"!hideErrors\"\n               (errorChange)=\"onErrorChange($event)\">\n</bb-form-error>\n\n<!-- The file picker hint. -->\n<p *ngIf=\"hint as hintContent\"\n   class=\"bb-file-picker-hint\">\n    <ng-template [bbTemplate]=\"hintContent\">\n        {{ hintContent }}\n    </ng-template>\n</p>\n\n<!-- Upload icon. -->\n<ng-template #uploadIcon>\n    <svg xmlns=\"http://www.w3.org/2000/svg\"\n         class=\"bb-file-picker-icon\"\n         width=\"24\"\n         height=\"24\">\n        <path fill=\"currentColor\"\n              d=\"M2 12.5C2 9.46 4.46 7 7.5 7H18c2.21 0 4 1.79 4 4s-1.79 4-4 4H9.5a2.5 2.5 0 010-5H17v2H9.41c-.55 0-.55 1 0 1H18c1.1 0 2-.9 2-2s-.9-2-2-2H7.5C5.57 9 4 10.57 4 12.5S5.57 16 7.5 16H17v2H7.5C4.46 18 2 15.54 2 12.5z\"></path>\n        <path fill=\"none\" d=\"M0 0h24v24H0V0z\"></path>\n    </svg>\n</ng-template>\n\n<!-- Close icon. -->\n<ng-template #closeIcon>\n    <svg xmlns=\"http://www.w3.org/2000/svg\"\n         class=\"bb-file-picker-icon\"\n         width=\"24\"\n         height=\"24\">\n        <path fill=\"currentColor\"\n              d=\"M19 6.41L17.59 5 12 10.59 6.41 5 5 6.41 10.59 12 5 17.59 6.41 19 12 13.41 17.59 19 19 17.59 13.41 12z\"></path>\n        <path d=\"M0 0h24v24H0z\" fill=\"none\"></path>\n    </svg>\n</ng-template>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: forwardRef(() => BbFilePicker),
                        multi: true
                    },
                    {
                        provide: NG_VALIDATORS,
                        useExisting: BbFilePicker,
                        multi: true
                    }
                ],
                host: {
                    'class': 'bb-file-picker',
                    '[class.required]': 'required',
                    '[class.disabled]': 'disabled',
                    '[class.grouped]': 'grouped',
                    '[class.hovered]': 'hovered',
                    '[class.error]': 'error'
                },
                animations: [
                    trigger('bbFilePickerImageAnimation', [
                        transition(':enter', [
                            style({ opacity: 0 }),
                            animate('200ms', style({ opacity: 1 }))
                        ]),
                        transition(':leave', [
                            animate('200ms', style({ opacity: 0 }))
                        ])
                    ])
                ],
                inputs: ['disabled', 'hideErrors', 'grouped', 'required'],
                preserveWhitespaces: false,
                styles: [".bb-file-picker{display:block}.bb-file-picker.disabled>.bb-file-picker-wrapper{background-color:hsla(0,0%,80%,.64);cursor:default}.bb-file-picker.grouped{margin-bottom:20px}.bb-file-picker.full>.bb-file-picker-wrapper{width:100%}.bb-file-picker:not(.disabled) .bb-file-picker-button{background-color:#fff}.bb-file-picker:not(.disabled).hovered>.bb-file-picker-wrapper,.bb-file-picker:not(.disabled):focus>.bb-file-picker-wrapper{box-shadow:inset 0 3px 0 0 rgba(183,188,194,.2),0 6px 6px -6px rgba(0,0,0,.1),0 0 0 3px rgba(0,123,255,.2)}.bb-file-picker.required>.bb-file-picker-label:after{color:#c23934;content:\"*\";font-size:12px;line-height:1.5}.bb-file-picker.error>.bb-file-picker-label{color:#c23934}.bb-file-picker.error>.bb-file-picker-hint{display:none}.bb-file-picker.error>.bb-file-picker-wrapper{background-color:rgba(194,57,52,.25)!important;border:1px solid #962b26;box-shadow:inset 0 3px 0 0 rgba(183,188,194,.2),0 6px 6px -6px rgba(0,0,0,.1),0 0 0 3px rgba(194,57,52,.4)!important}.bb-file-picker.error .bb-file-picker-button{background-color:transparent}.bb-file-picker-wrapper{-moz-user-select:none;-ms-user-select:none;-webkit-user-select:none;background-color:#fff8ff;border:1px solid #b6bbc1;border-radius:3px;box-shadow:inset 0 3px 0 0 rgba(183,188,194,.2),0 6px 6px -6px rgba(0,0,0,.1);display:inline-flex;flex-direction:column;max-width:100%;transition:box-shadow .2s cubic-bezier(0,0,.2,1);user-select:none}.bb-file-picker-label{color:#000;display:block;margin-bottom:4px;position:relative}.bb-file-picker-input{height:.1px;opacity:0;overflow:hidden;position:absolute;width:.1px;z-index:-1}.bb-file-picker-container{display:inline-flex;min-height:38px}.bb-file-picker-button{-moz-appearance:none;-webkit-appearance:none;align-items:center;appearance:none;background-color:hsla(0,0%,100%,.5);border:none;border-bottom-left-radius:3px;border-right:1px solid #b6bbc1;border-top-left-radius:3px;color:#2196f3;cursor:pointer;display:inline-flex;font-weight:500;line-height:1;margin:0;padding:0 12px;transition:box-shadow .2s cubic-bezier(0,0,.2,1);white-space:nowrap;z-index:1}.bb-file-picker-button:focus{box-shadow:0 0 0 3px rgba(0,123,255,.2)}.bb-file-picker-button:disabled{cursor:default}.bb-file-picker-button.destructive{color:#f55656}.bb-file-picker-button.standalone{border-bottom-right-radius:3px;border-right:none;border-top-right-radius:3px;width:100%}.bb-file-picker-icon{margin-right:4px}.bb-file-picker-zone{flex:1;font-size:14px;justify-content:center;line-height:18px;padding:8px 12px}.bb-file-picker-info,.bb-file-picker-zone{align-items:center;color:#848f99;display:inline-flex}.bb-file-picker-info{background-color:#fff8ff;border-top:0 solid #b6bbc1;height:0;line-height:36px;overflow:hidden;padding:0 12px;transition:height .2s cubic-bezier(0,0,.2,1)}.bb-file-picker-info.visible{border-top-width:1px;height:38px}.bb-file-picker-info>span{font-size:14px;max-width:100%;opacity:.4;overflow:hidden;text-overflow:ellipsis;white-space:nowrap}.bb-file-picker-info>span:first-child{flex:1}.bb-file-picker-info>span:last-child{margin-left:12px}.bb-file-picker-image-container{border-top:1px solid #b6bbc1;overflow:hidden}.bb-file-picker-image-wrapper{margin:20px auto;max-width:40%;width:100%}.bb-file-picker-image{background-position:50%;background-repeat:no-repeat;background-size:cover;border-radius:4px;display:flex;overflow:hidden;position:relative;width:100%;will-change:opacity}.bb-file-picker-hint{color:#738694;display:block;font-size:13px;margin-top:4px}"]
            },] }
];
BbFilePicker.propDecorators = {
    fileInput: [{ type: ViewChild, args: ['fileInput', { static: true },] }],
    extraTemplate: [{ type: ContentChild, args: ['extra',] }],
    label: [{ type: Input }],
    hint: [{ type: Input }],
    accept: [{ type: Input }],
    showImages: [{ type: Input }],
    allowDragging: [{ type: Input }],
    value: [{ type: Input }],
    valueChange: [{ type: Output }],
    onDrag: [{ type: HostListener, args: ['dragover', ['$event', 'true'],] }, { type: HostListener, args: ['dragleave', ['$event', 'false'],] }],
    onDrop: [{ type: HostListener, args: ['drop', ['$event'],] }]
};

class BbDatePickerBase {
}
const BbFilePickerMixinBase$1 = mixinRequired(mixinReadonly(mixinDisabled(mixinGrouped(mixinError(mixinHideErrors(BbDatePickerBase))))));
class BbDatePicker extends BbFilePickerMixinBase$1 {
    constructor(_localeId) {
        super();
        this._localeId = _localeId;
        // Readonly data.
        this._yearRegex = /^(10|[1-9][0-9])\d{2}$/; // Note: 1000 - 9999
        // Inputs.
        this.label = null;
        this.hint = null;
        this.dayPlaceholder = 'dd';
        this.monthPlaceholder = 'mm';
        this.yearPlaceholder = 'yyyy';
        this.trackByValue = (_, item) => item === null || item === void 0 ? void 0 : item.value;
        this.valueChange = new EventEmitter();
        // Subscriptions.
        this._subscription = new Subscription();
        this.onTouchedCallback = () => {
        };
        this.onChangeCallback = () => {
        };
        this.formToDateString = ({ year, month, day }) => {
            var _a;
            if ((_a = this.form) === null || _a === void 0 ? void 0 : _a.invalid) {
                return null;
            }
            const prefixedMonth = this.padString(`${month}`, 2, '0');
            const prefixedDay = this.padString(`${day}`, 2, '0');
            return [year, prefixedMonth, prefixedDay].join('-');
        };
        this.getDaysInMonth = (month = null, year = null) => {
            var _a;
            if (((_a = this.form) === null || _a === void 0 ? void 0 : _a.invalid) || month === null || year === null) {
                return 31;
            }
            const parsedYear = parseInt(year, 10);
            return new Date(parsedYear, month, 0).getDate();
        };
        this.isValidDateString = (value) => {
            return typeof value === 'string' && /^[0-9]{4}-[0-9]{2}-[0-9]{2}$/.test(value);
        };
        this.padString = (value, maxLength, fillString = '0') => {
            const count = maxLength - value.length;
            for (let index = 0; index < count; index++) {
                value = `${fillString}${value}`;
            }
            return value;
        };
    }
    get value() {
        var _a;
        return this.formToDateString((_a = this.form) === null || _a === void 0 ? void 0 : _a.value);
    }
    set value(value) {
        this.parseDateStringInForm(value);
        this.valueChange.emit(this.value);
    }
    ngOnInit() {
        this.composeForm();
        this.getData();
        this.handleChanges();
    }
    ngOnDestroy() {
        var _a;
        (_a = this._subscription) === null || _a === void 0 ? void 0 : _a.unsubscribe();
    }
    writeValue(value) {
        this.parseDateStringInForm(value);
    }
    registerOnChange(method) {
        this.onChangeCallback = method;
    }
    registerOnTouched(method) {
        this.onTouchedCallback = method;
    }
    setDisabledState(isDisabled) {
        var _a, _b;
        this.disabled = isDisabled;
        if (this.disabled) {
            (_a = this.form) === null || _a === void 0 ? void 0 : _a.disable();
        }
        else {
            (_b = this.form) === null || _b === void 0 ? void 0 : _b.enable();
        }
    }
    onErrorChange(error) {
        this.error = !!error;
    }
    validate({ value }) {
        if (value === null || value === undefined) {
            return null;
        }
        const data = this.parseDateString(value);
        const date = new Date(data === null || data === void 0 ? void 0 : data.year, (data === null || data === void 0 ? void 0 : data.month) - 1, data === null || data === void 0 ? void 0 : data.day);
        const maxDays = this.getDaysInMonth(data === null || data === void 0 ? void 0 : data.month, `${data === null || data === void 0 ? void 0 : data.year}`);
        const isInvalidDate = date.toString() === 'Invalid Date';
        const exceededMaxDays = (data === null || data === void 0 ? void 0 : data.day) > maxDays;
        return (isInvalidDate || exceededMaxDays) && {
            invalidDate: true
        };
    }
    handleChanges() {
        const dateOrNull$ = this.form.valueChanges.pipe(map(value => this.formToDateString(value)), distinctUntilChanged());
        const subscription = dateOrNull$.subscribe(() => {
            this.valueChange.emit(this.value);
            this.onChangeCallback(this.value);
        });
        this._subscription.add(subscription);
    }
    getData() {
        const months$ = this.getMonthData();
        const days$ = this.getDayData();
        this.data$ = combineLatest([months$, days$]).pipe(map(([months, days]) => ({ months, days })), tap(data => { var _a; return this.verifyMaxDays((_a = data === null || data === void 0 ? void 0 : data.days) === null || _a === void 0 ? void 0 : _a.length); }));
    }
    verifyMaxDays(maxDays) {
        const control = this.form.get('day');
        if ((control === null || control === void 0 ? void 0 : control.value) === null || (control === null || control === void 0 ? void 0 : control.value) <= maxDays) {
            return;
        }
        control.patchValue(maxDays);
    }
    getDayData() {
        var _a;
        const count$ = this.form.valueChanges.pipe(startWith((_a = this.form) === null || _a === void 0 ? void 0 : _a.value), map(({ month, year }) => this.getDaysInMonth(month, year)), distinctUntilChanged());
        return count$.pipe(map(count => Array(count).fill(0).map((_, index) => ({
            label: `${index + 1}`,
            value: index + 1
        }))));
    }
    getMonthData() {
        const array = Array(12).fill(0).map((_, month) => {
            const date = new Date(2000, month, 1);
            const label = formatDate(date, 'LLLL', this._localeId);
            return { label, value: month + 1 };
        });
        return of(array);
    }
    composeForm() {
        const { required, pattern, minLength, maxLength } = Validators;
        const yearPattern = pattern(this._yearRegex);
        const yearValidators = [required, yearPattern, minLength(4), maxLength(4)];
        this.form = new FormGroup({
            day: new FormControl(null, required),
            month: new FormControl(null, required),
            year: new FormControl(null, yearValidators)
        });
    }
    parseDateStringInForm(value) {
        var _a, _b;
        // If the date string was not a valid format, reset the form.
        if (!this.isValidDateString(value)) {
            if (value !== null) {
                (_a = this.form) === null || _a === void 0 ? void 0 : _a.reset({ year: null, month: null, day: null }, { emitEvent: false });
            }
            return;
        }
        // Parse the date string and save in the form.
        const data = this.parseDateString(value);
        const year = (data === null || data === void 0 ? void 0 : data.year) !== null && (data === null || data === void 0 ? void 0 : data.year) !== undefined
            ? this.padString(`${data === null || data === void 0 ? void 0 : data.year}`, 4, '0')
            : null;
        const formData = Object.assign(Object.assign({}, (data !== null && data !== void 0 ? data : {})), { year });
        (_b = this.form) === null || _b === void 0 ? void 0 : _b.patchValue(formData, { emitEvent: false });
    }
    parseDateString(value) {
        if (!this.isValidDateString(value)) {
            return null;
        }
        // Parse the date string.
        const [year, month, day] = value === null || value === void 0 ? void 0 : value.split('-');
        return {
            day: day ? parseInt(day, 10) : null,
            month: month ? parseInt(month, 10) : null,
            year: year ? parseInt(year, 10) : null
        };
    }
}
BbDatePicker.decorators = [
    { type: Component, args: [{
                selector: 'bb-date-picker',
                template: "<!-- Label of the date picker. -->\n<label *ngIf=\"label as labelContent\"\n       class=\"bb-date-picker-label\">\n    <ng-template [bbTemplate]=\"labelContent\">\n        {{ labelContent }}\n    </ng-template>\n</label>\n\n<!-- The form containing the year/month/day fields. -->\n<form *ngIf=\"data$ | async as data\"\n      [formGroup]=\"form\"\n      class=\"bb-date-picker-container\"\n      novalidate>\n    <div class=\"bb-date-picker-item\">\n        <bb-form-control hideErrors>\n            <select [class.has-value]=\"!!form?.get('day')?.value\"\n                    bbInput\n                    autocomplete=\"off\"\n                    formControlName=\"day\">\n                <option [ngValue]=\"null\">{{ dayPlaceholder }}</option>\n                <option *ngFor=\"let day of data?.days; trackBy: trackByValue\"\n                        [ngValue]=\"day?.value\">\n                    {{ day?.label }}\n                </option>\n            </select>\n        </bb-form-control>\n    </div>\n    <div class=\"bb-date-picker-item\">\n        <bb-form-control hideErrors>\n            <select [class.has-value]=\"!!form?.get('month')?.value\"\n                    bbInput\n                    autocomplete=\"off\"\n                    formControlName=\"month\">\n                <option [ngValue]=\"null\">{{ monthPlaceholder }}</option>\n                <option *ngFor=\"let month of data?.months; trackBy: trackByValue\"\n                        [ngValue]=\"month?.value\">\n                    {{ month?.label }}\n                </option>\n            </select>\n        </bb-form-control>\n    </div>\n    <div class=\"bb-date-picker-item\">\n        <bb-form-control hideErrors>\n            <input [placeholder]=\"yearPlaceholder\"\n                   bbInput\n                   type=\"text\"\n                   inputmode=\"numeric\"\n                   minlength=\"4\"\n                   maxlength=\"4\"\n                   pattern=\"^[0-9]{4}$\"\n                   formControlName=\"year\"\n                   autocomplete=\"off\">\n        </bb-form-control>\n    </div>\n</form>\n\n<!-- The date picker error. -->\n<bb-form-error *ngIf=\"!hideErrors\"\n               (errorChange)=\"onErrorChange($event)\">\n</bb-form-error>\n\n<!-- The date picker hint. -->\n<p *ngIf=\"hint as hintContent\"\n   class=\"bb-date-picker-hint\">\n    <ng-template [bbTemplate]=\"hintContent\">\n        {{ hintContent }}\n    </ng-template>\n</p>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: forwardRef(() => BbDatePicker),
                        multi: true
                    },
                    {
                        provide: NG_VALIDATORS,
                        useExisting: BbDatePicker,
                        multi: true
                    }
                ],
                host: {
                    'class': 'bb-date-picker',
                    '[class.required]': 'required',
                    '[class.readonly]': 'readonly',
                    '[class.disabled]': 'disabled',
                    '[class.grouped]': 'grouped',
                    '[class.error]': 'error'
                },
                inputs: ['required', 'readonly', 'disabled', 'grouped', 'hideErrors'],
                preserveWhitespaces: false,
                styles: [".bb-date-picker{display:block}.bb-date-picker.required>.bb-date-picker-label:after{color:#c23934;content:\"*\";font-size:12px;line-height:1.5}.bb-date-picker.readonly{pointer-events:none}.bb-date-picker.readonly .bb-form-control-container{background-color:#f9f9f9!important;border:1px dotted #bdc4c9;cursor:default}.bb-date-picker.readonly .bb-form-control-container>select:disabled{opacity:1}.bb-date-picker.grouped{margin-bottom:20px}.bb-date-picker.error>.bb-date-picker-label{color:#c23934}.bb-date-picker.error .bb-form-control-container{background-color:rgba(194,57,52,.25)!important;border:1px solid #962b26;box-shadow:inset 0 3px 0 0 rgba(183,188,194,.2),0 6px 6px -6px rgba(0,0,0,.1),0 0 0 3px rgba(194,57,52,.4)!important}.bb-date-picker-label{color:#000;display:block;font-size:16px;font-weight:500;margin-bottom:4px}.bb-date-picker-container{display:flex}.bb-date-picker-item{flex:1}.bb-date-picker-item>bb-form-control.focused{position:relative;z-index:1}.bb-date-picker-item>.bb-form-control>.bb-form-control-container>select:not(.has-value),.bb-date-picker-item>.bb-form-control>.bb-form-control-container>select>option:first-child{color:#d2d2d2}.bb-date-picker-item>.bb-form-control>.bb-form-control-container>select:not(.has-value)>option:not(:first-child){color:#000}.bb-date-picker-item:not(:first-child):not(:last-child) .bb-form-control-container{border-radius:0!important}.bb-date-picker-item:first-child .bb-form-control-container{border-bottom-right-radius:0!important;border-right:none!important;border-top-right-radius:0!important}.bb-date-picker-item:last-child .bb-form-control-container{border-bottom-left-radius:0!important;border-left:none!important;border-top-left-radius:0!important}.bb-date-picker-hint{color:#738694;display:block;font-size:13px;margin-top:4px;pointer-events:all}"]
            },] }
];
BbDatePicker.ctorParameters = () => [
    { type: String, decorators: [{ type: Inject, args: [LOCALE_ID,] }] }
];
BbDatePicker.propDecorators = {
    label: [{ type: Input }],
    hint: [{ type: Input }],
    dayPlaceholder: [{ type: Input }],
    monthPlaceholder: [{ type: Input }],
    yearPlaceholder: [{ type: Input }],
    value: [{ type: Input }],
    valueChange: [{ type: Output }]
};

class BbSuffix {
}
BbSuffix.decorators = [
    { type: Directive, args: [{
                selector: '[bbSuffix]',
                host: {
                    'class': 'bb-suffix'
                }
            },] }
];
class BbPrefix {
}
BbPrefix.decorators = [
    { type: Directive, args: [{
                selector: '[bbPrefix]',
                host: {
                    'class': 'bb-prefix'
                }
            },] }
];

class BbFormSubmit {
    constructor(_host, _formDirective) {
        this._host = _host;
        this._formDirective = _formDirective;
    }
    ngOnInit() {
        // Submit changes.
        const regularSubmitChanges$ = fromEvent(this.nativeElement, 'submit').pipe(mapTo(true));
        // Reactive form submit changes.
        const reactiveSubmitChanges$ = this.reactiveSubmitChanges();
        // Reset changes.
        const resetChanges$ = fromEvent(this.nativeElement, 'reset').pipe(mapTo(false));
        // Listen to the submit status.
        this.submitStatus$ = merge(regularSubmitChanges$, reactiveSubmitChanges$, resetChanges$).pipe(startWith(false), distinctUntilChanged(), shareReplay(1));
    }
    submit() {
        var _a, _b;
        (_b = (_a = this._formDirective) === null || _a === void 0 ? void 0 : _a.ngSubmit) === null || _b === void 0 ? void 0 : _b.emit();
    }
    reset() {
        var _a;
        return (_a = this.nativeElement) === null || _a === void 0 ? void 0 : _a.reset();
    }
    get nativeElement() {
        var _a;
        return (_a = this._host) === null || _a === void 0 ? void 0 : _a.nativeElement;
    }
    reactiveSubmitChanges() {
        if (!this._formDirective) {
            return EMPTY;
        }
        return this._formDirective.ngSubmit.pipe(mapTo(true));
    }
}
BbFormSubmit.decorators = [
    { type: Directive, args: [{
                selector: 'form',
                exportAs: 'bbForm'
            },] }
];
BbFormSubmit.ctorParameters = () => [
    { type: ElementRef },
    { type: FormGroupDirective, decorators: [{ type: Optional }, { type: Self }] }
];

class BbSelect {
    constructor(_elementRef, _ngControl, _formControl) {
        this._elementRef = _elementRef;
        this._ngControl = _ngControl;
        this._formControl = _formControl;
        this.hasArrow = true;
        this._disabled = false;
        this._required = false;
    }
    ngOnInit() {
        // Set an arrow on the form control if
        // it requires one.
        this._formControl.arrow = this.hasArrow;
    }
    ngDoCheck() {
        if (this._ngControl) {
            this.disabled = this._ngControl.disabled;
        }
    }
    ngOnDestroy() {
        var _a;
        (_a = this._subscription) === null || _a === void 0 ? void 0 : _a.unsubscribe();
    }
    focus() {
        var _a, _b;
        return (_b = (_a = this._elementRef) === null || _a === void 0 ? void 0 : _a.nativeElement) === null || _b === void 0 ? void 0 : _b.focus();
    }
    setFocus(hasFocus) {
        if (!this._formControl) {
            return;
        }
        this._formControl.focused = hasFocus;
    }
    get id() {
        var _a, _b;
        return (_b = (_a = this._formControl) === null || _a === void 0 ? void 0 : _a.labelId) !== null && _b !== void 0 ? _b : null;
    }
    get disabled() {
        if (this._ngControl && this._ngControl.disabled !== null) {
            return this._ngControl.disabled;
        }
        return this._disabled;
    }
    set disabled(newValue) {
        this._disabled = coerceBooleanProperty(newValue);
        if (!this._formControl) {
            return;
        }
        this._formControl.disabled = this._disabled;
    }
    get required() {
        return this._required;
    }
    set required(newValue) {
        this._required = coerceBooleanProperty(newValue);
        if (!this._formControl) {
            return;
        }
        this._formControl.required = this._required;
    }
}
BbSelect.decorators = [
    { type: Directive, args: [{
                selector: 'select[bbInput]',
                host: {
                    '[disabled]': 'disabled',
                    '(focus)': 'setFocus(true)',
                    '(blur)': 'setFocus(false)'
                }
            },] }
];
BbSelect.ctorParameters = () => [
    { type: ElementRef },
    { type: NgControl, decorators: [{ type: Optional }, { type: Self }] },
    { type: BbFormControl, decorators: [{ type: Optional }, { type: Host }] }
];
BbSelect.propDecorators = {
    id: [{ type: HostBinding, args: ['attr.id',] }],
    disabled: [{ type: Input }],
    required: [{ type: Input }]
};
class BbInput extends BbSelect {
    constructor(_elementRef, _ngControl, _formControl) {
        super(_elementRef, _ngControl, _formControl);
        this._elementRef = _elementRef;
        this._ngControl = _ngControl;
        this._formControl = _formControl;
        this.hasArrow = false;
        this._readonly = false;
    }
    get readonly() {
        return this._readonly;
    }
    set readonly(newValue) {
        this._readonly = coerceBooleanProperty(newValue);
        if (!this._formControl) {
            return;
        }
        this._formControl.readonly = this._readonly;
    }
}
BbInput.decorators = [
    { type: Directive, args: [{
                selector: 'input[bbInput], textarea[bbInput]',
                host: {
                    '[disabled]': 'disabled',
                    '[readonly]': 'readonly',
                    '(focus)': 'setFocus(true)',
                    '(blur)': 'setFocus(false)'
                }
            },] }
];
BbInput.ctorParameters = () => [
    { type: ElementRef },
    { type: NgControl, decorators: [{ type: Optional }, { type: Self }] },
    { type: BbFormControl, decorators: [{ type: Optional }, { type: Host }] }
];
BbInput.propDecorators = {
    readonly: [{ type: Input }]
};

class BbFormGroupBase {
}
const BbFormGroupMixinBase = mixinGrouped(BbFormGroupBase);
class BbFormGroup extends BbFormGroupMixinBase {
}
BbFormGroup.decorators = [
    { type: Component, args: [{
                selector: 'bb-form-group',
                template: "<ng-content></ng-content>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                host: {
                    'class': 'bb-form-group',
                    '[class.grouped]': 'grouped'
                },
                inputs: ['grouped'],
                preserveWhitespaces: false,
                styles: [".bb-form-group{display:flex;flex-wrap:wrap;margin-top:-10px;width:100%}.bb-form-group>*{margin-top:10px}.bb-form-group>:not(:first-child){margin-left:10px}.bb-form-group.start{justify-content:flex-start}.bb-form-group.end{justify-content:flex-end}.bb-form-group.vertical{flex-direction:column;justify-content:normal}.bb-form-group.vertical>*{margin-left:0}.bb-form-group.vertical.start{align-items:flex-start}.bb-form-group.vertical.end{align-items:flex-end}.bb-form-group.grouped{margin-bottom:20px}"]
            },] }
];

class BbFormError {
    constructor(_form, _control, _errors) {
        this._form = _form;
        this._control = _control;
        this._errors = _errors;
        // Outputs.
        this.errorChange = new EventEmitter();
        // Subscriptions.
        this._ngControl = null;
    }
    // Inputs.
    set control(value) {
        this._ngControl = value;
    }
    get control() {
        return this._ngControl;
    }
    ngOnInit() {
        // Set the control if it was not set before.
        if (!this.control) {
            this.control = this._control;
        }
        // Listen for the submit changes
        const submitStatus$ = this.submitStatusChanges();
        // An observable that emits error changes.
        const errorChanges$ = this.statusChanges().pipe(map(() => this.getError(this.control.errors)), distinctUntilChanged(), map(error => typeof error === 'string' ? { token: error } : error));
        // Whenever the user submitted the form we want to
        // listen for any error changes.
        this.error$ = combineLatest([submitStatus$, errorChanges$]).pipe(map(([submitted, error]) => {
            // If not submitted return null.
            if (!submitted) {
                return null;
            }
            // Return the error if submitted.
            return error;
        }), tap(value => this.errorChange.emit(value)));
    }
    getError(errors) {
        // Validate the errors exist.
        if (!errors) {
            return null;
        }
        // Get the first error in the errors.
        const [errorName] = Object.keys(errors);
        const error = errors[errorName];
        // Validate the error exists.
        if (!error) {
            return null;
        }
        // Map the arrays into a object so we can
        // access the functions by a key-lookup.
        const errorFunctions = this._errors.reduce((previous, current) => (Object.assign(Object.assign({}, previous), current)), {});
        // Get the error function.
        const errorFunction = errorFunctions[errorName] || errorFunctions['unknown'];
        // Validate the error function exists.
        if (!errorFunction) {
            return null;
        }
        // Return the parsed error.
        return errorFunction(error);
    }
    submitStatusChanges() {
        if (!this._form) {
            return of(true);
        }
        return this._form.submitStatus$;
    }
    statusChanges() {
        if (!this.control) {
            return EMPTY;
        }
        if (!this.control.statusChanges) {
            return EMPTY;
        }
        return this.control.statusChanges.pipe(startWith(this.control.status), delay(0));
    }
}
BbFormError.decorators = [
    { type: Component, args: [{
                selector: 'bb-form-error',
                template: "<ng-container *ngIf=\"error$ | async as error\">\n    <p *ngIf=\"error?.token | bbLocalize:{optional: true, data: error?.data} as message\"\n       class=\"bb-form-error-message\">\n        {{ message }}\n    </p>\n</ng-container>",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                host: {
                    'class': 'bb-form-error'
                },
                preserveWhitespaces: false,
                styles: [".bb-form-error{display:block}.bb-form-error-message{color:#c23934;display:block;font-size:13px;margin-top:4px}"]
            },] }
];
BbFormError.ctorParameters = () => [
    { type: BbFormSubmit, decorators: [{ type: Optional }] },
    { type: NgControl, decorators: [{ type: Optional }] },
    { type: Array, decorators: [{ type: Optional }, { type: Inject, args: [ELEMENTS_ERRORS,] }] }
];
BbFormError.propDecorators = {
    control: [{ type: Input }],
    errorChange: [{ type: Output }]
};

class BbFocusTrap {
    constructor(_platform, _elementRef) {
        this._platform = _platform;
        this._elementRef = _elementRef;
        this._focusableElements = [
            'a[href]',
            'area[href]',
            'input:not([disabled]):not([type="hidden"]):not([aria-hidden])',
            'select:not([disabled]):not([aria-hidden])',
            'textarea:not([disabled]):not([aria-hidden])',
            'button:not([disabled]):not([aria-hidden])',
            'iframe',
            'object',
            'embed',
            '[contenteditable]',
            '[tabindex]:not([tabindex^="-"])'
        ];
        this.focus = (element) => {
            return element && element.focus && element.focus();
        };
        this.isTabEvent = (event) => {
            return event.key === 'Tab' || event.keyCode === 9;
        };
    }
    onKeydown(event) {
        // Validate it is a tab event.
        if (!this.isTabEvent(event)) {
            return;
        }
        // Trap the focus inside the element.
        return this.trapFocus(event);
    }
    trapFocus(event) {
        // Validate that the DOM is available.
        if (!this._platform.isBrowser) {
            return;
        }
        // Get all focusable nodes.
        const focusableNodes = this.getFocusableNodes();
        // Focus the first available element if the focus
        // is not in the modal.
        if (!this.element.contains(document.activeElement)) {
            return this.focus(focusableNodes[0]);
        }
        const focusedItemIndex = focusableNodes.indexOf(document.activeElement);
        if (event.shiftKey && focusedItemIndex === 0) {
            this.focus(focusableNodes[focusableNodes.length - 1]);
            return event.preventDefault();
        }
        if (!event.shiftKey && focusedItemIndex === focusableNodes.length - 1) {
            this.focus(focusableNodes[0]);
            return event.preventDefault();
        }
    }
    get element() {
        return this._elementRef.nativeElement;
    }
    getFocusableNodes() {
        const nodes = this.element.querySelectorAll(this._focusableElements);
        return Array(...nodes);
    }
}
BbFocusTrap.decorators = [
    { type: Directive, args: [{
                selector: '[bbFocusTrap]'
            },] }
];
BbFocusTrap.ctorParameters = () => [
    { type: Platform },
    { type: ElementRef }
];
BbFocusTrap.propDecorators = {
    onKeydown: [{ type: HostListener, args: ['keydown', ['$event'],] }]
};

class BbButtonBase {
    constructor(_elementRef) {
        this._elementRef = _elementRef;
    }
}
const BbButtonMixinBase = mixinDisabled(mixinLoad(BbButtonBase));
class BbButton extends BbButtonMixinBase {
    constructor(_elementRef) {
        super(_elementRef);
    }
    get nativeElement() {
        return this._elementRef.nativeElement;
    }
    focus() {
        this.nativeElement && this.nativeElement.focus();
    }
}
BbButton.decorators = [
    { type: Component, args: [{
                selector: 'button[bb-button]',
                template: "<bb-spinner [class.visible]=\"loading\"\n            class=\"bb-button-spinner small\">\n</bb-spinner>\n\n<ng-content></ng-content>\n",
                exportAs: 'bbButton',
                host: {
                    'class': 'bb-button',
                    '[class.loading]': 'loading',
                    '[attr.disabled]': 'disabled || loading || null'
                },
                inputs: ['disabled', 'loading'],
                encapsulation: ViewEncapsulation.None,
                changeDetection: ChangeDetectionStrategy.OnPush,
                styles: [".bb-button{-moz-user-select:none;-ms-user-select:none;-webkit-user-select:none;align-items:center;background-color:#fff;border:1px solid #d4d8e1;border-radius:4px;box-shadow:0 6px 6px -6px rgba(0,0,0,.1);color:#2d3c4d!important;cursor:pointer;display:inline-flex;font-size:16px;font-style:normal;font-weight:500;height:38px;justify-content:center;padding:0 8px;position:relative;text-align:center;text-decoration:none;transition-duration:.2s;transition-property:padding-left,background-color,box-shadow;transition-timing-function:cubic-bezier(0,0,.2,1);user-select:none}.bb-button:hover{background-color:#fcfcfb}.bb-button:active,.bb-button:focus{box-shadow:0 6px 6px -6px rgba(0,0,0,.1),0 0 0 3px rgba(220,224,233,.5)}.bb-button.full{width:100%}.bb-button.full.loading{padding-left:42px}.bb-button.primary{background-color:#5b53ff;border-color:#2f2b85;color:#fff!important}.bb-button.primary:hover{background-color:#564ee6}.bb-button.primary:active,.bb-button.primary:focus{box-shadow:0 6px 6px -6px rgba(0,0,0,.1),0 0 0 3px rgba(91,83,255,.5)}.bb-button.destructive{background-color:#d32f2f;border-color:#8f2020;color:#fff!important}.bb-button.destructive:hover{background-color:#c32929}.bb-button.destructive:active,.bb-button.destructive:focus{box-shadow:0 6px 6px -6px rgba(0,0,0,.1),0 0 0 3px rgba(211,47,47,.5)}.bb-button.loading,.bb-button.loading:active,.bb-button.loading:focus,.bb-button.loading:hover,.bb-button:disabled,.bb-button:disabled:active,.bb-button:disabled:focus,.bb-button:disabled:hover,.bb-button[disabled],.bb-button[disabled]:active,.bb-button[disabled]:focus,.bb-button[disabled]:hover{box-shadow:none;cursor:default;opacity:.4}.bb-button-spinner{opacity:0;overflow:hidden;padding:0!important;transition:width .2s cubic-bezier(0,0,.2,1),opacity .1s cubic-bezier(0,0,.2,1);visibility:hidden;width:0}.bb-button-spinner.visible{margin-right:8px;opacity:1;visibility:visible;width:20px}.bb-button.loading.full>.bb-button-spinner{left:8px;margin-right:0;position:absolute}"]
            },] }
];
BbButton.ctorParameters = () => [
    { type: ElementRef }
];
class BbAnchor extends BbButton {
    constructor(_elementRef) {
        super(_elementRef);
    }
    stopDisabledEvents(event) {
        // If the button is not disabled (and not loading) let all events pass.
        if (!this.disabled && !this.loading) {
            return;
        }
        // If the button is disabled and/or loading stop
        // the event from bubbling up.
        event.preventDefault();
        event.stopImmediatePropagation();
    }
}
BbAnchor.decorators = [
    { type: Component, args: [{
                selector: 'a[bb-button]',
                template: "<bb-spinner [class.visible]=\"loading\"\n            class=\"bb-button-spinner small\">\n</bb-spinner>\n\n<ng-content></ng-content>\n",
                exportAs: 'bbButton, bbAnchor',
                host: {
                    'class': 'bb-button',
                    '[attr.tabindex]': '(disabled || loading) ? -1 : (tabIndex || 0)',
                    '[attr.disabled]': 'disabled || loading || null',
                    '[attr.aria-disabled]': '(disabled || loading).toString()',
                    '(click)': 'stopDisabledEvents($event)'
                },
                inputs: ['disabled', 'loading'],
                encapsulation: ViewEncapsulation.None,
                changeDetection: ChangeDetectionStrategy.OnPush,
                styles: [".bb-button{-moz-user-select:none;-ms-user-select:none;-webkit-user-select:none;align-items:center;background-color:#fff;border:1px solid #d4d8e1;border-radius:4px;box-shadow:0 6px 6px -6px rgba(0,0,0,.1);color:#2d3c4d!important;cursor:pointer;display:inline-flex;font-size:16px;font-style:normal;font-weight:500;height:38px;justify-content:center;padding:0 8px;position:relative;text-align:center;text-decoration:none;transition-duration:.2s;transition-property:padding-left,background-color,box-shadow;transition-timing-function:cubic-bezier(0,0,.2,1);user-select:none}.bb-button:hover{background-color:#fcfcfb}.bb-button:active,.bb-button:focus{box-shadow:0 6px 6px -6px rgba(0,0,0,.1),0 0 0 3px rgba(220,224,233,.5)}.bb-button.full{width:100%}.bb-button.full.loading{padding-left:42px}.bb-button.primary{background-color:#5b53ff;border-color:#2f2b85;color:#fff!important}.bb-button.primary:hover{background-color:#564ee6}.bb-button.primary:active,.bb-button.primary:focus{box-shadow:0 6px 6px -6px rgba(0,0,0,.1),0 0 0 3px rgba(91,83,255,.5)}.bb-button.destructive{background-color:#d32f2f;border-color:#8f2020;color:#fff!important}.bb-button.destructive:hover{background-color:#c32929}.bb-button.destructive:active,.bb-button.destructive:focus{box-shadow:0 6px 6px -6px rgba(0,0,0,.1),0 0 0 3px rgba(211,47,47,.5)}.bb-button.loading,.bb-button.loading:active,.bb-button.loading:focus,.bb-button.loading:hover,.bb-button:disabled,.bb-button:disabled:active,.bb-button:disabled:focus,.bb-button:disabled:hover,.bb-button[disabled],.bb-button[disabled]:active,.bb-button[disabled]:focus,.bb-button[disabled]:hover{box-shadow:none;cursor:default;opacity:.4}.bb-button-spinner{opacity:0;overflow:hidden;padding:0!important;transition:width .2s cubic-bezier(0,0,.2,1),opacity .1s cubic-bezier(0,0,.2,1);visibility:hidden;width:0}.bb-button-spinner.visible{margin-right:8px;opacity:1;visibility:visible;width:20px}.bb-button.loading.full>.bb-button-spinner{left:8px;margin-right:0;position:absolute}"]
            },] }
];
BbAnchor.ctorParameters = () => [
    { type: ElementRef }
];
BbAnchor.propDecorators = {
    tabIndex: [{ type: Input }]
};

class BbTemplate {
    constructor(_templateRef, _viewContainerRef) {
        this._templateRef = _templateRef;
        this._viewContainerRef = _viewContainerRef;
    }
    set bbTemplate(content) {
        // Get the template.
        const template = content instanceof TemplateRef
            ? content
            : this._templateRef;
        // Clear the view container ref and create the view.
        this._viewContainerRef.clear();
        this._viewContainerRef.createEmbeddedView(template);
    }
}
BbTemplate.decorators = [
    { type: Directive, args: [{
                selector: '[bbTemplate]'
            },] }
];
BbTemplate.ctorParameters = () => [
    { type: TemplateRef },
    { type: ViewContainerRef }
];
BbTemplate.propDecorators = {
    bbTemplate: [{ type: Input }]
};

class BbAutosize {
    constructor(_renderer, _elementRef) {
        this._renderer = _renderer;
        this._elementRef = _elementRef;
        // Min/max heights for the textarea.
        this.minHeight = null;
        this.maxHeight = null;
        this.rows = 1;
    }
    get element() {
        var _a;
        return (_a = this._elementRef) === null || _a === void 0 ? void 0 : _a.nativeElement;
    }
    ngAfterViewInit() {
        // Update the styles after the DOM has loaded.
        this.updateStyles();
    }
    onWindowResize() {
        // Update the styles when the window is resized.
        this.updateStyles();
    }
    onInputReceived() {
        // Update the styles after the textarea received input.
        this.updateStyles();
    }
    updateStyles() {
        var _a, _b, _c;
        // Validate the element exists.
        if (!this.element) {
            return;
        }
        // Calculate border height which is not included in the scroll height.
        const borderHeight = ((_a = this.element) === null || _a === void 0 ? void 0 : _a.offsetHeight) - ((_b = this.element) === null || _b === void 0 ? void 0 : _b.clientHeight);
        // Reset textarea height to auto that correctly calculate the new height.
        this.setHeight('auto');
        // Set new height.
        this.setHeight(`${((_c = this.element) === null || _c === void 0 ? void 0 : _c.scrollHeight) + borderHeight}px`);
    }
    setHeight(value) {
        this._renderer.setStyle(this.element, 'height', value);
    }
}
BbAutosize.decorators = [
    { type: Directive, args: [{
                selector: 'textarea[bbAutosize]'
            },] }
];
BbAutosize.ctorParameters = () => [
    { type: Renderer2 },
    { type: ElementRef }
];
BbAutosize.propDecorators = {
    minHeight: [{ type: Input }, { type: HostBinding, args: ['style.min-height',] }],
    maxHeight: [{ type: Input }, { type: HostBinding, args: ['style.max-height',] }],
    rows: [{ type: Input }, { type: HostBinding, args: ['rows',] }],
    onWindowResize: [{ type: HostListener, args: ['window:resize',] }],
    onInputReceived: [{ type: HostListener, args: ['input',] }]
};

class BbRelativeTime {
    constructor(_localize, _localeId) {
        this._localize = _localize;
        this._localeId = _localeId;
        this.isValidDate = (date) => {
            return typeof date === 'object' &&
                typeof date.getTime === 'function' &&
                !isNaN(date.getTime());
        };
        this.parseAsDate = (value) => {
            // Add a Z after the date so
            // it is treated as a UTC date.
            if (typeof value === 'string' && /^[0-9]{4}-[0-9]{2}-[0-9]{2}T[0-9]{2}:[0-9]{2}:[0-9]{2}$/.test(value)) {
                return new Date(`${value}Z`);
            }
            // Return the regular parsed date.
            return new Date(value);
        };
        this.isToday = (date) => {
            const today = new Date();
            return (date === null || date === void 0 ? void 0 : date.getDate()) === (today === null || today === void 0 ? void 0 : today.getDate()) &&
                (date === null || date === void 0 ? void 0 : date.getMonth()) === (today === null || today === void 0 ? void 0 : today.getMonth()) &&
                (date === null || date === void 0 ? void 0 : date.getFullYear()) === (today === null || today === void 0 ? void 0 : today.getFullYear());
        };
        this.isYesterday = (date) => {
            var _a, _b;
            const yesterday = new Date((_a = new Date()) === null || _a === void 0 ? void 0 : _a.setDate(((_b = new Date()) === null || _b === void 0 ? void 0 : _b.getDate()) - 1));
            return (date === null || date === void 0 ? void 0 : date.getDate()) === (yesterday === null || yesterday === void 0 ? void 0 : yesterday.getDate()) &&
                (date === null || date === void 0 ? void 0 : date.getMonth()) === (yesterday === null || yesterday === void 0 ? void 0 : yesterday.getMonth()) &&
                (date === null || date === void 0 ? void 0 : date.getFullYear()) === (yesterday === null || yesterday === void 0 ? void 0 : yesterday.getFullYear());
        };
        this.isTomorrow = (date) => {
            var _a, _b;
            const tomorrow = new Date((_a = new Date()) === null || _a === void 0 ? void 0 : _a.setDate(((_b = new Date()) === null || _b === void 0 ? void 0 : _b.getDate()) + 1));
            return (date === null || date === void 0 ? void 0 : date.getDate()) === (tomorrow === null || tomorrow === void 0 ? void 0 : tomorrow.getDate()) &&
                (date === null || date === void 0 ? void 0 : date.getMonth()) === (tomorrow === null || tomorrow === void 0 ? void 0 : tomorrow.getMonth()) &&
                (date === null || date === void 0 ? void 0 : date.getFullYear()) === (tomorrow === null || tomorrow === void 0 ? void 0 : tomorrow.getFullYear());
        };
        this._datePipe = new DatePipe(this._localeId);
    }
    transform(value) {
        // Parse the value to a date object.
        const date = this.parseAsDate(value);
        // Check if the date is valid.
        if (!this.isValidDate(date)) {
            return value;
        }
        // Return the formatted date.
        return this.format(date);
    }
    format(date) {
        // It happens tomorrow.
        if (this.isTomorrow(date)) {
            return this.formatString(date, 'times.tomorrow_at', 'shortTime');
        }
        // It happened today.
        if (this.isToday(date)) {
            return this.formatString(date, 'times.today_at', 'shortTime');
        }
        // It happened yesterday.
        if (this.isYesterday(date)) {
            return this.formatString(date, 'times.yesterday_at', 'shortTime');
        }
        // Just return the date.
        return this.formatString(date);
    }
    formatString(date, token, format) {
        // Just return the parsed date.
        if (!this._localize || !token || !format) {
            return this._datePipe.transform(date, 'mediumDate');
        }
        // Return the translated formatting.
        return this._localize.translate({
            token: token,
            data: { date: this._datePipe.transform(date, format) }
        });
    }
}
BbRelativeTime.decorators = [
    { type: Pipe, args: [{
                name: 'bbRelativeTime'
            },] }
];
BbRelativeTime.ctorParameters = () => [
    { type: Localize, decorators: [{ type: Optional }] },
    { type: String, decorators: [{ type: Inject, args: [LOCALE_ID,] }] }
];

let nextUniqueId$2 = 0;
class BbCheckboxBase {
}
const BbCheckboxMixinBase = mixinDisabled(mixinGrouped(BbCheckboxBase));
class BbCheckbox extends BbCheckboxMixinBase {
    constructor(_changeDetection) {
        super();
        this._changeDetection = _changeDetection;
        // Data.
        this.labelId = `bb-checkbox-${nextUniqueId$2++}`;
        this._checked = false;
        // Methods.
        this.onChange = () => {
        };
        this.onTouched = () => {
        };
    }
    get checked() {
        return this._checked;
    }
    set checked(value) {
        this._checked = coerceBooleanProperty(value);
        this._changeDetection.markForCheck();
    }
    registerOnChange(fn) {
        this.onChange = fn;
    }
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    setDisabledState(isDisabled) {
        this.disabled = isDisabled;
        this._changeDetection.markForCheck();
    }
    writeValue(newValue) {
        this.checked = newValue;
    }
    onInteractionEvent(event) {
        // Stop the propagation to prevent it from bubbling up.
        event.stopPropagation();
    }
    onInputClick(event) {
        // Stop the propagation to prevent it from bubbling up.
        event.stopPropagation();
        // Toggle the value.
        this.checked = !this.checked;
        // Emit the change event.
        this.onChange && this.onChange(this.checked);
    }
}
BbCheckbox.decorators = [
    { type: Component, args: [{
                selector: 'bb-checkbox',
                template: "<input [id]=\"labelId\"\n       [checked]=\"checked\"\n       [disabled]=\"disabled\"\n       (change)=\"onInteractionEvent($event)\"\n       (click)=\"onInputClick($event)\"\n       class=\"bb-checkbox-input\"\n       tabindex=\"-1\"\n       type=\"checkbox\">\n\n<label [for]=\"labelId\"\n       [attr.tabindex]=\"disabled ? -1 : (tabIndex || 0)\"\n       (keyup.space)=\"onInputClick($event)\"\n       class=\"bb-checkbox-label\">\n    <span>\n        <svg width=\"12px\"\n             height=\"10px\"\n             viewbox=\"0 0 12 10\">\n            <polyline points=\"1.5 6 4.5 9 10.5 1\"></polyline>\n        </svg>\n    </span>\n    <span>\n        <ng-content></ng-content>\n    </span>\n</label>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                providers: [
                    { provide: NG_VALUE_ACCESSOR, useExisting: forwardRef(() => BbCheckbox), multi: true }
                ],
                host: {
                    'class': 'bb-checkbox',
                    '[class.grouped]': 'grouped'
                },
                inputs: ['grouped', 'disabled'],
                preserveWhitespaces: false,
                styles: [".bb-checkbox{display:block}.bb-checkbox+.bb-checkbox{margin-top:5px}.bb-checkbox.grouped{margin-bottom:20px}.bb-checkbox-input{display:none}.bb-checkbox-label{-moz-user-select:none;-ms-user-select:none;-webkit-user-select:none;cursor:pointer;display:flex;margin:0;user-select:none}.bb-checkbox-label>span{display:inline-block;transform:translateZ(0);vertical-align:middle}.bb-checkbox-label>span:first-child{border:1px solid #bdc4c9;border-radius:3px;height:18px;min-height:18px;min-width:18px;position:relative;transition:all .2s ease;vertical-align:middle;width:18px}.bb-checkbox-label>span:last-child{padding-left:8px}.bb-checkbox-label>span:first-child>svg{fill:none;left:2px;position:absolute;stroke:#fff;stroke-dasharray:16px;stroke-dashoffset:16px;stroke-linecap:round;stroke-linejoin:round;stroke-width:2;top:3px;transform:translateZ(0);transition:all .3s ease;transition-delay:.1s}.bb-checkbox-input:not(:disabled):not(:checked)+.bb-checkbox-label:focus>span:first-child{box-shadow:0 0 0 3px rgba(222,205,205,.5)}.bb-checkbox-input:not(:disabled):checked+.bb-checkbox-label:focus>span:first-child{box-shadow:0 0 0 3px rgba(91,83,255,.5)}.bb-checkbox-input:checked+.bb-checkbox-label>span:first-child{background-color:#5b53ff;border-color:#5b53ff}.bb-checkbox-input:checked+.bb-checkbox-label>span:first-child>svg{stroke-dashoffset:0}.bb-checkbox-input:disabled+.bb-checkbox-label{cursor:default;pointer-events:none}.bb-checkbox-input:disabled:checked+.bb-checkbox-label>span:first-child{background-color:hsla(0,0%,80%,.64);border-color:hsla(0,0%,80%,.64)}"]
            },] }
];
BbCheckbox.ctorParameters = () => [
    { type: ChangeDetectorRef }
];
BbCheckbox.propDecorators = {
    tabIndex: [{ type: Input }],
    checked: [{ type: Input }]
};

class BbSpinnerBase {
}
const BbSpinnerMixinBase = mixinDisabled(BbSpinnerBase);
class BbSpinner extends BbSpinnerMixinBase {
    constructor() {
        super(...arguments);
        // Inputs.
        this.color = '#303f9f';
        this.alt = 'Loading...';
    }
}
BbSpinner.decorators = [
    { type: Component, args: [{
                selector: 'bb-spinner',
                template: "<!--\n    The spinner element.\n\n    This element contains a circle with a 25% bar that spins\n    indefinitely. The border color can be set via the \"color\"\n    property.\n-->\n<div class=\"bb-spinner-container\">\n    <div [style.border-left-color]=\"color\"\n         [class.disabled]=\"disabled\"\n         class=\"bb-spinner-loader\"\n         aria-live=\"assertive\"\n         role=\"alert\">\n        {{ alt }}\n    </div>\n    <ng-content select=\"img\"></ng-content>\n</div>\n\n<!--\n    The content of the spinner.\n\n    This element contains the content of the spinner. The user\n    can set the content inside the tag.\n-->\n<span class=\"bb-spinner-content\">\n    <ng-content></ng-content>\n</span>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                host: { 'class': 'bb-spinner' },
                inputs: ['color', 'alt', 'disabled'],
                preserveWhitespaces: false,
                styles: [".bb-spinner{align-items:center;color:#565656;display:inline-flex;flex-direction:column;justify-content:center;padding:20px 0;text-align:center}.bb-spinner.block{display:flex}.bb-spinner.inverse .bb-spinner-loader{border-color:hsla(0,0%,100%,.15)}.bb-spinner.inverse .bb-spinner-content{color:hsla(0,0%,100%,.5)}.bb-spinner.small .bb-spinner-loader,.bb-spinner.small .bb-spinner-loader:after{height:20px;min-height:20px;min-width:20px;width:20px}.bb-spinner.small .bb-spinner-loader{border-width:2px}.bb-spinner.small .bb-spinner-content{font-size:12px}.bb-spinner.medium .bb-spinner-loader,.bb-spinner.medium .bb-spinner-loader:after{height:60px;min-height:60px;min-width:60px;width:60px}.bb-spinner.medium .bb-spinner-loader{border-width:4px}.bb-spinner.medium .bb-spinner-content{font-size:16px}.bb-spinner.large .bb-spinner-loader,.bb-spinner.large .bb-spinner-loader:after{height:100px;min-height:100px;min-width:100px;width:100px}.bb-spinner.large .bb-spinner-loader{border-width:6px}.bb-spinner.large .bb-spinner-content{font-size:20px}.bb-spinner.horizontal{flex-direction:row;text-align:left}.bb-spinner.horizontal .bb-spinner-content:not(:empty){margin-left:10px;margin-top:0}.bb-spinner.vertical{flex-direction:column;text-align:center}.bb-spinner.vertical .bb-spinner-content:not(:empty){margin-left:0;margin-top:10px}.bb-spinner-container{display:flex;position:relative}.bb-spinner-container>img{height:60%;left:20%;position:absolute;top:20%;width:60%}.bb-spinner-loader{-webkit-animation:spin 1s linear infinite;animation:spin 1s linear infinite;border:4px solid #f2f4f6;color:transparent;display:inline-block;margin:0;overflow:hidden;padding:0;position:relative;text-indent:100%;vertical-align:middle}.bb-spinner-loader,.bb-spinner-loader:after{border-radius:50%;height:60px;min-height:60px;min-width:60px;width:60px}.bb-spinner-loader.disabled{-moz-user-select:none;-ms-user-select:none;-webkit-animation-play-state:paused;-webkit-user-select:none;animation-play-state:paused;cursor:default;pointer-events:none;user-select:none}.bb-spinner-content:not(:empty){font-size:16px;font-weight:500;margin-top:10px}@-webkit-keyframes spin{0%{transform:rotate(0deg)}to{transform:rotate(1turn)}}@keyframes spin{0%{transform:rotate(0deg)}to{transform:rotate(1turn)}}"]
            },] }
];
BbSpinner.propDecorators = {
    color: [{ type: Input }],
    alt: [{ type: Input }]
};

class BbFocus {
    constructor(_zone, _platform, _elementRef) {
        this._zone = _zone;
        this._platform = _platform;
        this._elementRef = _elementRef;
    }
    get nativeElement() {
        return this._elementRef.nativeElement;
    }
    ngAfterViewInit() {
        // Run the method outside the Angular zone.
        this._zone.runOutsideAngular(() => this.focus());
    }
    focus() {
        // Check if set timeout exists and the user is
        // using the site on desktop devices.
        if (!this._platform.isDesktop || !setTimeout) {
            return;
        }
        // Check if the element and the focus method exist, if so focus the element.
        if (!this.nativeElement || !this.nativeElement.focus) {
            return;
        }
        // Execute the focus method in a timeout.
        setTimeout(() => this.nativeElement.focus(), 0);
    }
}
BbFocus.decorators = [
    { type: Directive, args: [{
                selector: '[bbFocus]'
            },] }
];
BbFocus.ctorParameters = () => [
    { type: NgZone },
    { type: Platform },
    { type: ElementRef }
];

class BbFileImage {
    constructor(_platform, _image, _sanitizer) {
        this._platform = _platform;
        this._image = _image;
        this._sanitizer = _sanitizer;
    }
    transform(file) {
        return __awaiter(this, void 0, void 0, function* () {
            // If the file is not a file return.
            if (!this._platform.isBrowser || !(file instanceof File)) {
                return null;
            }
            return this._image.fileToImage(file, [
                'image/png',
                'image/jpeg',
                'image/gif',
                'image/svg+xml'
            ]).then(image => {
                if (!image) {
                    return null;
                }
                return {
                    background: this._sanitizer.bypassSecurityTrustStyle(`url(${image.src})`),
                    aspectRatio: image.height / image.width
                };
            }).catch(() => {
                return null;
            });
        });
    }
}
BbFileImage.decorators = [
    { type: Pipe, args: [{
                name: 'bbFileImage'
            },] }
];
BbFileImage.ctorParameters = () => [
    { type: Platform },
    { type: ImageConverter },
    { type: DomSanitizer }
];

class BbAvatar {
    constructor() {
        // Inputs.
        this.size = 40;
        // State.
        this.state$ = new BehaviorSubject('initials');
        // Data.
        this._src = null;
        this._title = null;
        this._initials = '!';
        this._color = 'rgb(255, 255, 255)';
        this._backgroundColor = 'rgb(66, 66, 66)';
        this.toRGb = (value) => {
            const baseRed = 128;
            const baseGreen = 128;
            const baseBlue = 128;
            let seed = value.charCodeAt(0) ^ value.charCodeAt(1);
            const rand_1 = Math.abs((Math.sin(seed++) * 10000)) % 256;
            const rand_2 = Math.abs((Math.sin(seed++) * 10000)) % 256;
            const rand_3 = Math.abs((Math.sin(seed++) * 10000)) % 256;
            const r = Math.round((rand_1 + baseRed) / 2);
            const g = Math.round((rand_2 + baseGreen) / 2);
            const b = Math.round((rand_3 + baseBlue) / 2);
            return { r, g, b };
        };
        this.getInitials = (name) => {
            // Get all the initials.
            const names = (name || '').split(' ');
            const initials = names.map(name => name.charAt(0).toUpperCase());
            // Check if there are any initials.
            if (initials.length <= 0) {
                return null;
            }
            // Get the first and last initials.
            if (initials.length > 1) {
                return `${initials[0]}${initials[initials.length - 1]}`;
            }
            // Get only the first initial.
            return initials[0];
        };
        this.parseColorFormat = (value) => {
            var _a;
            // Hex 3 values.
            let match = value.match(/^#([0-9a-f]{3})$/i);
            if (match && match[1]) {
                const r = parseInt(match[1].charAt(0), 16) * 0x11;
                const g = parseInt(match[1].charAt(1), 16) * 0x11;
                const b = parseInt(match[1].charAt(2), 16) * 0x11;
                return { r, g, b };
            }
            // Hex 6 values.
            match = value.match(/^#([0-9a-f]{6})$/i);
            if (match && match[1]) {
                const r = parseInt(match[1].substr(0, 2), 16);
                const g = parseInt(match[1].substr(2, 2), 16);
                const b = parseInt(match[1].substr(4, 2), 16);
                return { r, g, b };
            }
            // Rgb values.
            match = value.match(/^rgb\s*\(\s*(\d+)\s*,\s*(\d+)\s*,\s*(\d+)\s*\)$/i);
            if (match) {
                const r = parseInt(match[1], 10);
                const g = parseInt(match[2], 10);
                const b = parseInt(match[3], 10);
                return { r, g, b };
            }
            // Not a supported color format.
            (_a = console === null || console === void 0 ? void 0 : console.warn) === null || _a === void 0 ? void 0 : _a.call(console, `BbAvatar: Unsupported tintColor "${value}" was used.`);
            return null;
        };
    }
    set src(value) {
        this._src = value;
        if (this._src) {
            this.state$.next('image');
        }
    }
    set title(value) {
        // Set the title and the initials.
        this._title = value || '!';
        this._initials = this.getInitials(this._title);
        // Generate the colors based on the name.
        const { r, g, b } = this.toRGb(this._title);
        this.setColors(r, g, b);
    }
    set tintColor(value) {
        const result = this.parseColorFormat(value);
        if (!result) {
            return;
        }
        const { r, g, b } = result;
        this.setColors(r, g, b);
    }
    get src() {
        return this._src;
    }
    get color() {
        return this._color;
    }
    get backgroundColor() {
        return this._backgroundColor;
    }
    get title() {
        return this._title;
    }
    get initials() {
        return this._initials;
    }
    setColors(red, green, blue) {
        this._color = `rgb(${red},${green},${blue})`;
        this._backgroundColor = `rgba(${red},${green},${blue}, 0.25)`;
    }
}
BbAvatar.decorators = [
    { type: Component, args: [{
                selector: 'bb-avatar',
                template: "<ng-container [ngSwitch]=\"state$ | async\">\n    <!-- Image. -->\n    <ng-container *ngSwitchCase=\"'image'\">\n        <img [src]=\"src\"\n             [alt]=\"initials\"\n             (error)=\"state$?.next('initials')\"\n             class=\"bb-avatar-image\">\n    </ng-container>\n\n    <!-- Default initials fallback. -->\n    <ng-container *ngSwitchDefault>\n        {{ initials }}\n    </ng-container>\n</ng-container>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                preserveWhitespaces: false,
                host: {
                    'class': 'bb-avatar',
                    'role': 'img',
                    '[style.width.px]': 'size',
                    '[style.height.px]': 'size',
                    '[style.font-size.px]': 'size / 2',
                    '[style.color]': 'color',
                    '[style.background-color]': 'backgroundColor'
                },
                styles: [".bb-avatar{-moz-user-select:none;-ms-user-select:none;-webkit-user-select:none;align-items:center;border-radius:50%;display:inline-flex;font-size:18px;font-weight:500;justify-content:center;line-height:1;overflow:hidden;position:relative;user-select:none}.bb-avatar.rounded{border-radius:4px}.bb-avatar.square{border-radius:0}.bb-avatar-image{-o-object-fit:cover;height:100%;object-fit:cover;position:absolute;width:100%}"]
            },] }
];
BbAvatar.propDecorators = {
    size: [{ type: Input }],
    src: [{ type: Input }],
    title: [{ type: Input }, { type: HostBinding, args: ['attr.title',] }],
    tintColor: [{ type: Input }]
};

class BbFileSize {
    constructor() {
        this.format = (value, decimals = 2) => {
            if (value === 0) {
                return '0 Bytes';
            }
            const k = 1024;
            const dm = decimals < 0 ? 0 : decimals;
            const sizes = ['Bytes', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'];
            const index = Math.floor(Math.log(value) / Math.log(k));
            return parseFloat((value / Math.pow(k, index)).toFixed(dm)) + ' ' + sizes[index];
        };
    }
    transform(value) {
        return this.format(value);
    }
}
BbFileSize.decorators = [
    { type: Pipe, args: [{
                name: 'bbFileSize'
            },] }
];

class BbIcon {
    constructor(_registry) {
        this._registry = _registry;
        // Data.
        this._src = null;
        // State.
        this.state$ = new BehaviorSubject(null);
        this.alt = null;
        this.size = 24;
        this._ariaHidden = true;
        this.warn = (content) => {
            return console && console.warn && console.warn(content);
        };
    }
    set src(newValue) {
        this.state$.next(this.convert(newValue));
        this._src = newValue;
    }
    get src() {
        return this._src;
    }
    set ariaHidden(newValue) {
        this._ariaHidden = coerceBooleanProperty(newValue);
    }
    get ariaHidden() {
        return this._ariaHidden;
    }
    convert(src) {
        const [namespace, value] = (src || '').split(':');
        const onError = () => {
            this.warn(`ElementsModule: The icon with src "${src}" could not be rendered correctly; Did you put a correct src attribute?`);
            return { type: 'unknown' };
        };
        if (!namespace || !value || !this._registry) {
            return onError();
        }
        // Try to find a image icon.
        const imageIcon = this.findImageIcon(namespace, value);
        if (imageIcon) {
            return imageIcon;
        }
        // Try to find a font icon.
        const fontIcon = this.findFontIcon(namespace, value);
        if (fontIcon) {
            return fontIcon;
        }
        // Return the on error function.
        return onError();
    }
    findImageIcon(namespace, value) {
        const data = this._registry.find(item => item['namespace'] === namespace && item['name'] === value);
        if (!data) {
            return null;
        }
        return { type: data['type'], src: data['src'] };
    }
    findFontIcon(namespace, content) {
        const data = this._registry.find(item => item['namespace'] === namespace);
        if (!data) {
            return null;
        }
        const uses = data['uses'];
        if (uses === 'data-attribute') {
            return { type: data['type'], className: data['className'], attribute: content };
        }
        return { type: data['type'], className: data['className'], content };
    }
}
BbIcon.decorators = [
    { type: Component, args: [{
                selector: 'bb-icon',
                template: "<ng-container *ngIf=\"state$ | async as data\"\n              [ngSwitch]=\"data?.type\">\n    <i *ngSwitchCase=\"'font'\"\n       [attr.data-icon]=\"data?.attribute\"\n       [class]=\"data?.className\">\n        {{ data?.content }}\n    </i>\n    <img *ngSwitchCase=\"'image'\"\n         [src]=\"data?.src\"\n         [alt]=\"alt\">\n    <svg *ngSwitchCase=\"'unknown'\"\n         viewBox=\"0 0 1 1\">\n        <rect x=\"0\" y=\"0\" width=\"1\" height=\"1\" fill=\"#c5c5c5\"></rect>\n    </svg>\n</ng-container>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                host: {
                    'class': 'bb-icon',
                    '[attr.aria-hidden]': 'ariaHidden',
                    '[style.width.px]': 'size',
                    '[style.height.px]': 'size',
                    '[style.min-width.px]': 'size',
                    '[style.min-height.px]': 'size',
                    '[style.font-size.px]': 'size'
                },
                preserveWhitespaces: false,
                styles: [".bb-icon{-moz-user-select:none;-ms-user-select:none;-webkit-user-select:none;align-items:center;display:flex;justify-content:center;user-select:none}.bb-icon>i{font-size:inherit}.bb-icon>img,.bb-icon>svg{display:flex;height:100%;width:100%}"]
            },] }
];
BbIcon.ctorParameters = () => [
    { type: Array, decorators: [{ type: Optional }, { type: Inject, args: [ELEMENTS_ICONS,] }] }
];
BbIcon.propDecorators = {
    alt: [{ type: Input }],
    size: [{ type: Input }],
    src: [{ type: Input }],
    ariaHidden: [{ type: Input }]
};

class BbTag {
    constructor() {
        // Outputs.
        this.onClose = new EventEmitter();
    }
}
BbTag.decorators = [
    { type: Component, args: [{
                selector: 'bb-tag',
                template: "<ng-content></ng-content>\n\n<button *ngIf=\"onClose?.observers?.length > 0\"\n        (click)=\"onClose.emit()\"\n        type=\"button\">\n    Close\n</button>",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                preserveWhitespaces: false,
                host: { 'class': 'bb-tag' },
                styles: [".bb-tag{background-color:#fafafa;border:1px solid #d9d9d9;border-radius:4px;color:rgba(0,0,0,.65);cursor:default;display:inline-block;font-feature-settings:\"tnum\";font-size:12px;font-variant:tabular-nums;line-height:20px;list-style:none;padding:0 7px;white-space:nowrap}.bb-tag:not(:last-child){margin-right:8px}"]
            },] }
];
BbTag.propDecorators = {
    onClose: [{ type: Output }]
};

const DECLARATIONS_EXPORTS = [
    // Directives.
    BbTemplate,
    BbFormError,
    BbFormSubmit,
    BbInput,
    BbSelect,
    BbFocus,
    BbFocusTrap,
    BbAutosize,
    BbPrefix,
    BbSuffix,
    // Pipes.
    BbFileSize,
    BbFileImage,
    BbRelativeTime,
    // Components.
    BbButton,
    BbAnchor,
    BbSpinner,
    BbIcon,
    BbFormControl,
    BbFormGroup,
    BbCheckbox,
    BbDropdown,
    BbDropdownItem,
    BbTag,
    BbAvatar,
    BbFilePicker,
    BbImagePicker,
    BbDatePicker
];
class ElementsModule {
    static forRoot() {
        return {
            ngModule: ElementsModule,
            providers: [
                { provide: ELEMENTS_ERRORS, useFactory: getErrors, multi: true },
                ELEMENTS_MATERIAL_ICON
            ]
        };
    }
}
ElementsModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule, ReactiveFormsModule, LocalizeModule],
                declarations: [...DECLARATIONS_EXPORTS],
                exports: [...DECLARATIONS_EXPORTS]
            },] }
];
const ɵ0 = { namespace: 'material', className: 'material-icons', uses: 'content', type: 'font' };
const ELEMENTS_MATERIAL_ICON = {
    provide: ELEMENTS_ICONS,
    useValue: ɵ0,
    multi: true
};
function getErrors() {
    const data = {
        unknown: () => 'form-control-errors.unknown',
        required: () => 'form-control-errors.required',
        pattern: () => 'form-control-errors.pattern',
        email: () => 'form-control-errors.email',
        emailTaken: () => 'form-control-errors.emailTaken',
        passwordMatch: () => 'form-control-errors.passwordMatch',
        confirm: () => 'form-control-errors.confirm',
        fullNumber: () => 'form-control-errors.fullNumber',
        min: ({ min }) => ({ token: 'form-control-errors.min', data: { min } }),
        max: ({ max }) => ({ token: 'form-control-errors.max', data: { max } }),
        maxFileSize: ({ maxSize }) => ({ token: 'form-control-errors.maxFileSize', data: { maxSize } }),
        minDate: ({ date }) => ({ token: 'form-control-errors.minDate', data: { date } }),
        maxDate: ({ date }) => ({ token: 'form-control-errors.maxDate', data: { date } }),
        invalidDate: () => 'form-control-errors.invalidDate',
        invalidFileType: () => 'form-control-errors.invalidFileType',
        minlength: ({ requiredLength, actualLength }) => ({
            token: 'form-control-errors.minlength',
            data: { requiredLength, actualLength }
        }),
        maxlength: ({ requiredLength, actualLength }) => ({
            token: 'form-control-errors.maxlength',
            data: { requiredLength, actualLength }
        })
    };
    return data;
}

/**
 * Generated bundle index. Do not edit.
 */

export { BbAnchor, BbAutosize, BbAvatar, BbButton, BbCheckbox, BbDatePicker, BbDropdown, BbDropdownItem, BbFileImage, BbFilePicker, BbFileSize, BbFocus, BbFocusTrap, BbFormControl, BbFormError, BbFormGroup, BbFormSubmit, BbIcon, BbImagePicker, BbInput, BbPrefix, BbRelativeTime, BbSelect, BbSpinner, BbSuffix, BbTag, BbTemplate, ELEMENTS_ERRORS, ELEMENTS_ICONS, ELEMENTS_MATERIAL_ICON, ElementsModule, getErrors, ɵ0 };
//# sourceMappingURL=bravobit-bb-foundation-elements.js.map
